from gooey import Gooey, GooeyParser

import matplotlib.pyplot as plt # hangs when imported here

@Gooey
def main():
    argparser = GooeyParser()
    argparser.add_argument('foo')

    args = argparser.parse_args()

    # import matplotlib.pyplot as plt # works as expected when imported here

    print(args.foo)


if __name__ == '__main__':
    main()


import wx
from gooey.gui.components.widgets.bases import BaseWidget
from gooey.gui.lang.i18n import _
from gooey.gui.util import wx_util
from gooey.gui.components.widgets import CheckBox
from gooey.util.functional import getin, findfirst, merge


class RadioGroup(BaseWidget):

    def __init__(self, parent, widgetInfo, *args, **kwargs):
        super(RadioGroup, self).__init__(parent, *args, **kwargs)
        self._parent = parent
        self.info = widgetInfo
        self._id = widgetInfo['id']
        self._options = widgetInfo['options']
        self.widgetInfo = widgetInfo
        self.error = wx.StaticText(self, label='')
        self.radioButtons = self.createRadioButtons()
        self.selected = None
        self.widgets = self.createWidgets()
        self.arrange()


        for button in self.radioButtons:
            button.Bind(wx.EVT_LEFT_DOWN, self.handleButtonClick)

        initialSelection = getin(self.info, ['options', 'initial_selection'], None)
        if initialSelection is not None:
            self.selected = self.radioButtons[initialSelection]
            self.selected.SetValue(True)
        self.handleImplicitCheck()

        self.applyStyleRules()


    def getValue(self):
        for button, widget in zip(self.radioButtons, self.widgets):
            if button.GetValue():  # is Checked
                return merge(widget.getValue(), {'id': self._id})
        else:
            # just return the first widget's value even though it's
            # not active so that the expected interface is satisfied
            return self.widgets[0].getValue()

    def setErrorString(self, message):
        for button, widget in zip(self.radioButtons, self.widgets):
            if button.GetValue():  # is Checked
                widget.setErrorString(message)
        self.Layout()

    def showErrorString(self, b):
        for button, widget in zip(self.radioButtons, self.widgets):
            if button.GetValue():  # is Checked
                widget.showErrorString(b)


    def arrange(self, *args, **kwargs):
        title = getin(self.widgetInfo, ['options', 'title'], _('choose_one'))
        if getin(self.widgetInfo, ['options', 'show_border'], False):
            boxDetails = wx.StaticBox(self, -1, title)
            boxSizer = wx.StaticBoxSizer(boxDetails, wx.VERTICAL)
        else:
            title = wx_util.h1(self, title)
            title.SetForegroundColour(self._options['label_color'])
            boxSizer = wx.BoxSizer(wx.VERTICAL)
            boxSizer.AddSpacer(10)
            boxSizer.Add(title, 0)

        for btn, widget in zip(self.radioButtons, self.widgets):
            sizer = wx.BoxSizer(wx.HORIZONTAL)
            sizer.Add(btn,0, wx.RIGHT, 4)
            sizer.Add(widget, 1, wx.EXPAND)
            boxSizer.Add(sizer, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(boxSizer)


    def handleButtonClick(self, event):
        currentSelection = self.selected
        nextSelection = event.EventObject

        if not self.isSameRadioButton(currentSelection, nextSelection):
            self.selected = nextSelection
            self.selected.SetValue(True)
        else:
            # user clicked on an already enabled radio button.
            # if it is not in the required section, allow it to be deselected
            if not self.widgetInfo['required']:
                self.selected.SetValue(False)
                self.selected = None

        self.applyStyleRules()
        self.handleImplicitCheck()


    def isSameRadioButton(self, radioButton1, radioButton2):
        return (getattr(radioButton1, 'Id', 'r1-not-found') ==
                getattr(radioButton2, 'Id', 'r2-not-found'))


    def applyStyleRules(self):
        """
        Conditionally disabled/enables form fields based on the current
        section in the radio group
        """
        # for reasons I have been completely unable to figure out
        # or understand, IFF you've interacted with one of the radio Buttons's
        # child components, then the act of disabling that component will
        # reset the state of the radioButtons thus causing it to forget
        # what should be selected. So, that is why we're collected the initial
        # state of all the buttons and resetting each button's state as we go.
        # it's wonky as hell
        states = [x.GetValue() for x in self.radioButtons]
        for button, selected, widget in zip(self.radioButtons, states, self.widgets):
            if isinstance(widget, CheckBox):
                widget.hideInput()
            if not selected: # not checked
                widget.Disable()
            else:
                widget.Enable()
            button.SetValue(selected)


    def handleImplicitCheck(self):
        """
        Checkboxes are hidden when inside of a RadioGroup as a selection of
        the Radio button is an implicit selection of the Checkbox. As such, we have
        to manually "check" any checkbox as needed.
        """
        for button, widget in zip(self.radioButtons, self.widgets):
            if isinstance(widget, CheckBox):
                if button.GetValue(): # checked
                    widget.setValue(True)
                else:
                    widget.setValue(False)


    def createRadioButtons(self):
        # button groups in wx are statefully determined via a style flag
        # on the first button (what???). All button instances are part of the
        # same group until a new button is created with the style flag RG_GROUP
        # https://wxpython.org/Phoenix/docs/html/wx.RadioButton.html
        # (What???)
        firstButton = wx.RadioButton(self, style=wx.RB_GROUP)
        firstButton.SetValue(False)
        buttons = [firstButton]

        for _ in getin(self.widgetInfo, ['data','widgets'], [])[1:]:
            buttons.append(wx.RadioButton(self))
        return buttons

    def createWidgets(self):
        """
        Instantiate the Gooey Widgets that are used within the RadioGroup
        """
        from gooey.gui.components import widgets
        widgets = [getattr(widgets, item['type'])(self, item)
                   for item in getin(self.widgetInfo, ['data', 'widgets'], [])]
        # widgets should be disabled unless
        # explicitly selected
        for widget in widgets:
            widget.Disable()
        return widgets



import wx
import wx.lib.agw.multidirdialog as MDD
import os
import re

from gooey.gui.components.widgets.core.text_input import TextInput
from gooey.gui.components.widgets.dialogs.calender_dialog import CalendarDlg
from gooey.gui.components.widgets.dialogs.time_dialog import TimeDlg
from gooey.gui.lang.i18n import _
from gooey.util.functional import merge
from gooey.gui.util.filedrop import FileDrop


class Chooser(wx.Panel):
    """
    TODO: Tests!
    TODO: Document GooeyOptions!
    Base 'Chooser' type.

    Launches a Dialog box that allows the user to pick files, directories,
    dates, etc.. and places the result into a TextInput in the UI

    TODO: oh, young me. DRY != Good Abstraction
    TODO: this is another weird inheritance hierarchy that's hard
          to follow. Why do subclasses reach into, not their parent
          class, but their _physical_ UI parent to grab the Gooey Options?
          All this could be simplified to make the data flow
          more apparent.
    """
    _gooey_options = {
        'pathsep': str
    }
    def __init__(self, parent, *args, **kwargs):
        super(Chooser, self).__init__(parent)
        self.options = parent._options
        buttonLabel = kwargs.pop('label', _('browse'))
        self.widget = TextInput(self, *args, **kwargs)
        self.button = wx.Button(self, label=buttonLabel)
        self.button.Bind(wx.EVT_BUTTON, self.spawnDialog)
        self.dropTarget = FileDrop(self.widget, self.dropHandler)
        self.widget.SetDropTarget(self.dropTarget)
        self.layout()

    def dropHandler(self, x, y, filenames):
        sep = self.options.get('pathsep', os.pathsep)
        self.widget.setValue(sep.join(filenames))
        return True

    def layout(self):
        layout = wx.BoxSizer(wx.HORIZONTAL)
        layout.Add(self.widget, 1, wx.EXPAND | wx.TOP, 2)
        layout.Add(self.button, 0, wx.LEFT, 10)

        v = wx.BoxSizer(wx.VERTICAL)
        v.Add(layout, 1, wx.EXPAND, wx.TOP, 1)
        self.SetSizer(v)

    def spawnDialog(self, event):
        fd = self.getDialog()
        if fd.ShowModal() == wx.ID_CANCEL:
            return
        self.processResult(self.getResult(fd))

    def getDialog(self):
        return wx.FileDialog(self, _('open_file'))

    def getResult(self, dialog):
        return dialog.GetPath()

    def processResult(self, result):
        self.setValue(result)

    def setValue(self, value):
        self.widget.setValue(value)

    def SetHint(self, value):
        self.widget.SetHint(value)

    def getValue(self):
        return self.widget.getValue()


class FileChooser(Chooser):
    """ Retrieve an existing file from the system """
    def getDialog(self):
        options = self.Parent._options
        return wx.FileDialog(self, message=options.get('message', _('open_file')),
                             style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST,
                             defaultFile=options.get('default_file', _("enter_filename")),
                             defaultDir=options.get('default_dir', _('')),
                             wildcard=options.get('wildcard', wx.FileSelectorDefaultWildcardStr))


class MultiFileChooser(Chooser):
    """ Retrieve an multiple files from the system """
    def getDialog(self):
        options = self.Parent._options
        return wx.FileDialog(self, message=options.get('message', _('open_files')),
                             style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST | wx.FD_MULTIPLE,
                             defaultFile=options.get('default_file', _("enter_filename")),
                             defaultDir=options.get('default_dir', _('')),
                             wildcard=options.get('wildcard', wx.FileSelectorDefaultWildcardStr))

    def getResult(self, dialog):
        return os.pathsep.join(dialog.GetPaths())


class FileSaver(Chooser):
    """ Specify the path to save a new file """
    def getDialog(self):
        options = self.Parent._options
        return wx.FileDialog(
            self,
            style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
            defaultFile=options.get('default_file', _("enter_filename")),
            defaultDir=options.get('default_dir', _('')),
            message=options.get('message', _('choose_file')),
            wildcard=options.get('wildcard', wx.FileSelectorDefaultWildcardStr)
        )


class DirChooser(Chooser):
    """ Retrieve a path to the supplied directory """
    def getDialog(self):
        options = self.Parent._options
        return wx.DirDialog(self, message=options.get('message', _('choose_folder')),
                            defaultPath=options.get('default_path', os.getcwd()))

class MultiDirChooser(Chooser):
    """ Retrieve multiple directories from the system """
    def getDialog(self):
        options = self.Parent._options
        return MDD.MultiDirDialog(self,
                                  message=options.get('message', _('choose_folders')),
                                  title=_('choose_folders_title'),
                                  defaultPath=options.get('default_path', os.getcwd()),
                                  agwStyle=MDD.DD_MULTIPLE | MDD.DD_DIR_MUST_EXIST)
    def getResult(self, dialog):
        paths = dialog.GetPaths()
        # Remove volume labels from Windows paths
        if 'nt' == os.name:
            for i, path in enumerate(paths):
                if path:
                    parts = path.split(os.sep)
                    vol = parts[0]
                    drives = re.match(r'.*\((?P<drive>\w:)\)', vol)
                    paths[i] = os.sep.join([drives.group('drive')] + parts[1:])

        return os.pathsep.join(paths)


class DateChooser(Chooser):
    """ Launches a date picker which returns an ISO Date """
    def __init__(self, *args, **kwargs):
        defaults = {'label': _('choose_date')}
        super(DateChooser, self).__init__(*args, **merge(kwargs, defaults))

    def getDialog(self):
        return CalendarDlg(self)


class TimeChooser(Chooser):
    """ Launches a time picker which returns and ISO Time """
    def __init__(self, *args, **kwargs):
        defaults = {'label': _('choose_time')}
        super(TimeChooser, self).__init__(*args, **merge(kwargs, defaults))

    def getDialog(self):
        return TimeDlg(self)


class ColourChooser(Chooser):
    """ Launches a color picker which returns a hex color code"""
    def __init__(self, *args, **kwargs):
        defaults = {'label': _('choose_colour'),
                    'style': wx.TE_RICH}
        super(ColourChooser, self).__init__(*args, **merge(kwargs, defaults))

    def setValue(self, value):
        colour = wx.Colour(value)
        self.widget.widget.SetForegroundColour(colour)
        self.widget.widget.SetBackgroundColour(colour)
        self.widget.setValue(value)

    def getResult(self, dialog):
        colour = dialog.GetColourData().GetColour()

        # Set text box back/foreground to selected colour
        self.widget.widget.SetForegroundColour(colour)
        self.widget.widget.SetBackgroundColour(colour)

        return colour.GetAsString(wx.C2S_HTML_SYNTAX)

    def getDialog(self):
        return wx.ColourDialog(self)



#
# 11/30/2003 - Jeff Grimmett (grimmtooth@softhome.net)
#
# o Some issues with the listbox example; I tried correcting
#   it but it's still not working the way it should. Commented
#   out for now, as I found it.
#

import wx


class Foo(wx.Frame):
    def __init__(self, *args, **kwargs):
        super(Foo, self).__init__(*args, **kwargs)
        # self.panel = TestPanel(self, void)
        # self.Show()

class TestPopup(wx.PopupWindow):
    """Adds a bit of text and mouse movement to the wx.PopupWindow"""

    def __init__(self, parent, style):
        wx.PopupWindow.__init__(self, parent, style)
        pnl = self.pnl = wx.Panel(self)
        pnl.SetBackgroundColour("CADET BLUE")

        st = wx.StaticText(pnl, -1,
                           "This is a special kind of top level\n"
                           "window that can be used for\n"
                           "popup menus, combobox popups\n"
                           "and such.\n\n"
                           "Try positioning the demo near\n"
                           "the bottom of the screen and \n"
                           "hit the button again.\n\n"
                           "In this demo this window can\n"
                           "be dragged with the left button\n"
                           "and closed with the right."
                           ,
                           pos=(10, 10))

        sz = st.GetBestSize()
        self.SetSize((sz.width + 20, sz.height + 20))
        pnl.SetSize((sz.width + 20, sz.height + 20))

        pnl.Bind(wx.EVT_LEFT_DOWN, self.OnMouseLeftDown)
        pnl.Bind(wx.EVT_MOTION, self.OnMouseMotion)
        pnl.Bind(wx.EVT_LEFT_UP, self.OnMouseLeftUp)
        pnl.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)

        st.Bind(wx.EVT_LEFT_DOWN, self.OnMouseLeftDown)
        st.Bind(wx.EVT_MOTION, self.OnMouseMotion)
        st.Bind(wx.EVT_LEFT_UP, self.OnMouseLeftUp)
        st.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)

        wx.CallAfter(self.Refresh)

    def OnMouseLeftDown(self, evt):
        self.Refresh()
        self.ldPos = evt.GetEventObject().ClientToScreen(evt.GetPosition())
        self.wPos = self.ClientToScreen((0, 0))
        self.pnl.CaptureMouse()

    def OnMouseMotion(self, evt):
        if evt.Dragging() and evt.LeftIsDown():
            dPos = evt.GetEventObject().ClientToScreen(evt.GetPosition())
            nPos = (self.wPos.x + (dPos.x - self.ldPos.x),
                    self.wPos.y + (dPos.y - self.ldPos.y))
            self.Move(nPos)

    def OnMouseLeftUp(self, evt):
        if self.pnl.HasCapture():
            self.pnl.ReleaseMouse()

    def OnRightUp(self, evt):
        self.Show(False)
        self.Destroy()


class TestTransientPopup(wx.PopupTransientWindow):
    """Adds a bit of text and mouse movement to the wx.PopupWindow"""

    def __init__(self, parent, style, log):
        wx.PopupTransientWindow.__init__(self, parent, style)
        self.log = log
        panel = wx.Panel(self)
        panel.SetBackgroundColour("#FFB6C1")

        st = wx.StaticText(panel, -1,
                           "wx.PopupTransientWindow is a\n"
                           "wx.PopupWindow which disappears\n"
                           "automatically when the user\n"
                           "clicks the mouse outside it or if it\n"
                           "(or its first child) loses focus in \n"
                           "any other way.")
        btn = wx.Button(panel, -1, "Press Me")
        spin = wx.SpinCtrl(panel, -1, "Hello", size=(100, -1))
        btn.Bind(wx.EVT_BUTTON, self.OnButton)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(st, 0, wx.ALL, 5)
        sizer.Add(btn, 0, wx.ALL, 5)
        sizer.Add(spin, 0, wx.ALL, 5)
        panel.SetSizer(sizer)

        sizer.Fit(panel)
        sizer.Fit(self)
        self.Layout()

    def ProcessLeftDown(self, evt):
        self.log.write("ProcessLeftDown: %s\n" % evt.GetPosition())
        return wx.PopupTransientWindow.ProcessLeftDown(self, evt)

    def OnDismiss(self):
        self.log.write("OnDismiss\n")

    def OnButton(self, evt):
        btn = evt.GetEventObject()
        if btn.GetLabel() == "Press Me":
            btn.SetLabel("Pressed")
        else:
            btn.SetLabel("Press Me")


class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        wx.Panel.__init__(self, parent, -1)
        self.log = log

        b = wx.Button(self, -1, "Show wx.PopupWindow", (25, 50))
        self.Bind(wx.EVT_BUTTON, self.OnShowPopup, b)

        b = wx.Button(self, -1, "Show wx.PopupTransientWindow", (25, 95))
        self.Bind(wx.EVT_BUTTON, self.OnShowPopupTransient, b)

        # This isn't working so well, not sure why. Commented out for
        # now.

    #        b = wx.Button(self, -1, "Show wx.PopupWindow with listbox", (25, 140))
    #        self.Bind(wx.EVT_BUTTON, self.OnShowPopupListbox, b)

    def OnShowPopup(self, evt):
        # win = TestPopup(self.GetTopLevelParent(), wx.SIMPLE_BORDER)
        win = TestPopupWithListbox(self, wx.SIMPLE_BORDER, self.log)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = evt.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Show(True)

    def OnShowPopupTransient(self, evt):
        win = TestTransientPopup(self,
                                 wx.SIMPLE_BORDER,
                                 self.log)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = evt.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()

    def OnShowPopupListbox(self, evt):
        win = TestPopupWithListbox(self, wx.NO_BORDER, self.log)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = evt.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Show(True)


# This class is currently not implemented in the demo. It does not
# behave the way it should, so for the time being it's only here
# for show. If you figure out how to make it work, please send
# a corrected file to Robin!
class TestPopupWithListbox(wx.PopupWindow):
    def __init__(self, parent, style, log):
        wx.PopupWindow.__init__(self, parent, style)
        self.log = log
        import keyword
        self.lb = wx.ListCtrl(self, -1)
        self.lb.AppendColumn('Items')


        # sz = self.lb.GetBestSize()
        self.SetSize((150, 75))  # sz)
        self.lb.SetSize(self.GetClientSize())
        self.lb.SetFocus()
        self.Bind(wx.EVT_LISTBOX, self.OnListBox)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnListBoxDClick)

    def OnListBox(self, evt):
        obj = evt.GetEventObject()
        self.log.write("OnListBox: %s\n" % obj)
        self.log.write('Selected: %s\n' % obj.GetString(evt.GetInt()))
        evt.Skip()

    def OnListBoxDClick(self, evt):
        self.Hide()
        self.Destroy()


def void(*args, **kwargs):
    print(args, kwargs)

# ---------------------------------------------------------------------------

import wx.lib.inspection

def runTest():
    app = wx.App(False)
    frame = wx.Frame(None, -1, 'Demo PromptingComboBox Control and Work around',
                     size=(700, 400))
    win = TestPanel(frame, void)
    win.Show()
    wx.lib.inspection.InspectionTool().Show()
    frame.Show()
    app.MainLoop()


# ---------------------------------------------------------------------------



if __name__ == '__main__':
    runTest()


import wx


class FileDrop(wx.FileDropTarget):
    def __init__(self, window, dropStrategy=None):
        wx.FileDropTarget.__init__(self)
        self.window = window
        self.dropHandler = dropStrategy or self._defaultStrategy

    def OnDropFiles(self, x, y, filenames):
        return self.dropHandler(x, y, filenames)

    def _defaultStrategy(self, x, y, filenames):
        for name in filenames:
            self.window.WriteText(name)
        return True



import argparse
from gooey import Gooey, GooeyParser
from examples.issue458.resource import ConstantArray

def get_resource(num):
    return ConstantArray[num]

@Gooey
def main():
    parser = GooeyParser()
    parser.add_argument("FileChooser", widget="FileChooser",
                        gooey_options={
                            'wildcard':
                                "Comma separated file (*.csv)|*.csv|"
                                "All files (*.*)|*.*",
                            'default_dir': "c:/batch",
                            'default_file': "def_file.csv",
                            'message': "pick me"
                        }
                        )
    parser.add_argument("DirectoryChooser", widget="DirChooser",
                        gooey_options={
                            'wildcard':
                                "Comma separated file (*.csv)|*.csv|"
                                "All files (*.*)|*.*",
                            'message': "pick folder",
                            'default_path': r"C:\Users\Chris\Documents\books"
                        }
                        )
    parser.add_argument("FileSaver", widget="FileSaver",
                        gooey_options={
                            'wildcard':
                                "JPG (*.jpg)|*.jpg|"
                                "All files (*.*)|*.*",
                            'message': "pick folder",
                            'default_dir': "c:/projects",
                            'default_file': "def_file.csv"
                        }
                        )
    parser.add_argument("MultiFileSaver", widget="MultiFileChooser",
                        gooey_options={
                            'wildcard':
                                "Comma separated file (*.csv)|*.csv|"
                                "All files (*.*)|*.*",
                            'message': "pick folder",
                            'default_dir': "c:/temp",
                            'default_file': "def_file.csv"
                        }
                        )
    args = parser.parse_args()

    print(get_resource(int(args.ResourceNumber)))

if __name__ == "__main__":
    main()





import wx
import time

TIMER_ID1 = 2000
TIMER_ID2 = 2001


class MyForm(wx.Frame):

    def __init__(self):
        wx.Frame.__init__(self, None, wx.ID_ANY, "Timer Tutorial 2")

        # Add a panel so it looks the correct on all platforms
        panel = wx.Panel(self, wx.ID_ANY)

        self.timer = wx.Timer(self, id=TIMER_ID1)
        self.Bind(wx.EVT_TIMER, self.update, self.timer)
        self.timer2 = wx.Timer(self, id=TIMER_ID2)
        self.Bind(wx.EVT_TIMER, self.update, self.timer2)
        self.toggleBtn = wx.Button(panel, wx.ID_ANY, "Start Timer 1")
        self.toggleBtn.Bind(wx.EVT_BUTTON, self.onStartTimerOne)
        self.toggleBtn2 = wx.Button(panel, wx.ID_ANY, "Start Timer 2")
        self.toggleBtn2.Bind(wx.EVT_BUTTON, self.onStartTimerOne)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.toggleBtn, 0, wx.ALL | wx.CENTER, 5)
        sizer.Add(self.toggleBtn2, 0, wx.ALL | wx.CENTER, 5)
        panel.SetSizer(sizer)

    def onStartTimerOne(self, event):
        buttonObj = event.GetEventObject()
        btnLabel = buttonObj.GetLabel()
        timerNum = int(btnLabel[-1:])
        print(timerNum)

        if btnLabel == "Start Timer %s" % timerNum:
            if timerNum == 1:
                print("starting timer 1...")
                self.timer.Start(500)
            else:
                print("starting timer 2...")
                self.timer2.Start(3000)
            buttonObj.SetLabel("Stop Timer %s" % timerNum)
        else:
            if timerNum == 1:
                self.timer.Stop()
                print("timer 1 stopped!")
            else:
                self.timer2.Stop()
                print("timer 2 stopped!")
            buttonObj.SetLabel("Start Timer %s" % timerNum)

    def update(self, event):
        timerId = event.GetId()
        if timerId == TIMER_ID1:
            print("\ntimer 1 updated: ",)
        else:
            print("\ntimer 2 updated: ",)
        print(time.ctime())


# Run the program
if __name__ == "__main__":
    app = wx.PySimpleApp()
    frame = MyForm().Show()
    app.MainLoop()


from gooey import Gooey, GooeyParser
from argparse import ArgumentParser

def some_function(id_, len_):
    print(id_)
    print(len_)
    return

@Gooey(program_name = "Machine Learning Pipeline - Spatial Proximity Check",
       required_cols = 2,
       target=r'C:\Users\Chris\Dropbox\pretty_gui\Gooey\gooey\examples\issue615a.py',
       suppress_gooey_flag=True)
def main_spatial_proximity_gui():
    parser = GooeyParser(description = "Machine Learning Pipeline - Base Data Input")
    parser.add_argument('--pipeline_dataset', metavar = 'Pipeline Dataset (csv)', type = str, widget = 'FileChooser')
    args = parser.parse_args()




main_spatial_proximity_gui()



from gooey import Gooey, GooeyParser


@Gooey(tabbed_groups=True, default_size=(800, 600))
def create_arg_parser():
    arg_parser = GooeyParser()
    group = arg_parser.add_argument_group('Input/Output', gooey_options={'columns': 1})
    group.add_argument(dest='divisions_file', widget='FileChooser', help=(
        'The CSV with division information; see the Division Columns tab for details.'
    ))
    group.add_argument(dest='professors_file', widget='FileChooser',
        help='The CSV with professor information; see Professor Columns tab for details.',
    )
    group.add_argument(dest='students_file', widget='FileChooser',
        help='The CSV with student information; see the Student Columns tab for details.'
    )
    group.add_argument(dest='output_dir', default='~/Desktop/', widget='DirChooser',
        help='The folder to put the output.csv.',
    )
    return arg_parser


def main():
    arg_parser = create_arg_parser()
    arg_parser.parse_args()


if __name__ == '__main__':
    main()


import os
import time
from concurrent import futures

from gooey.gui.util.freeze import getResourcePath
from gooey.python_bindings import config_generator
from gooey.util.functional import merge


def run_integration(module, assertionFunction, **kwargs):
    """
    Integration test harness.

    WXPython is *super* finicky when it comes to integration tests. It needs
    the main Python thread for its app loop, which means we have to integration
    test on a separate thread. The causes further strangeness in how Unittest
    and WXPython interact. In short, each test must be in its own module and
    thus import its own wx instance, and be run in its own "space."

    So long as the above is satisfied, then integration tests can run reliably.

    """
    from gooey.gui import application
    options = merge({
        'image_dir': '::gooey/default',
        'language_dir': getResourcePath('languages'),
        'show_success_modal': False
    }, kwargs)
    module_path = os.path.abspath(module.__file__)
    parser = module.get_parser()
    build_spec = config_generator.create_from_parser(parser, module_path, **options)

    time.sleep(2)
    app = application.build_app(build_spec=build_spec)
    executor = futures.ThreadPoolExecutor(max_workers=1)
    # executor runs in parallel and will submit a wx.Destroy request
    # when done making its assertions
    testResult = executor.submit(assertionFunction, app, build_spec)
    # main loop blocks the main thread
    app.MainLoop()
    # .result() blocks as well while we wait for the thread to finish
    # any waiting it may be doing.
    testResult.result()
    del app





import wx
import sys
import time

from gooey import Gooey, GooeyParser, constants

bg_color = "#000000"
font_color = "#7CFC00"


@Gooey(
    #Background Settings
    # header_bg_color=bg_color,
    # body_bg_color=bg_color,
    # footer_bg_color=bg_color,
    # terminal_panel_color='#ffff00',
    # #Font Settings
    # terminal_font_family='Arial',
    # terminal_font_color=font_color,
    # terminal_font_weight=100
)
def gooey_app():
    parser = GooeyParser()

    parser.add_argument('-f', '--foo', help="Hello world!", gooey_options={
        'label_color': '#ffffff'
    })
    args = parser.parse_args()

    # Do something for a long time
    for i in range(1):
        print(time.time())
        print("HELLOO!!!!")
        time.sleep(1)

    return 0

if __name__ == '__main__':
    sys.exit(gooey_app())


from threading import Thread

from gooey import Gooey, GooeyParser
from argparse import SUPPRESS
import subprocess

cmd = [
    'ffmpeg',
    '-to', '30',
    '-i', 'Castle-in-the-Sky.mkv',
    'Castle-in-the-Sky.mp4'
]

@Gooey
def main():
    parser = GooeyParser()
    # parser.add_argument('filename1')
    args = parser.parse_args()

    process = subprocess.Popen(
        ' '.join(cmd),
        bufsize=1,
        stdout=subprocess.PIPE,
        stdin=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        universal_newlines=True,
        shell=True
    )
    for line in process.stdout:
        print(line)


if __name__ == '__main__':
    main()




import gooey


@gooey.Gooey
def main():
    parser = gooey.GooeyParser()
    parser.add_argument('--test', default=0, type=int, choices=[0, 1, 2, 3, 4],
                        help='This is a very, very, very long help text '
                             'to explain a very, very, very important input value. '
                             'Unfortunately, the end of this long message is cropped. ')
    args = parser.parse_args()
    print(args)


if __name__ == '__main__':
    main()


from gooey.gui.components.filtering.prefix_filter import PrefixTokenizers



def _include_layout_docs(f):
    """
    Combines the layout_options docsstring with the
    wrapped function's doc string.
    """
    f.__doc__ = (f.__doc__ or '') + LayoutOptions.__doc__
    return f


def _include_chooser_msg_wildcard_docs(f):
    """
    Combines the basic Chooser options (wildard, message) docsstring
    with the wrapped function's doc string.
    """
    _doc = """:param wildcard: Sets the wildcard, which can contain multiple file types, for 
                     example: "BMP files (.bmp)|.bmp|GIF files (.gif)|.gif"
    :param message:  Sets the message that will be displayed on the dialog.
    """
    f.__doc__ = (f.__doc__ or '') + _doc
    return f

def _include_choose_dir_file_docs(f):
    """
        Combines the basic Chooser options (wildard, message) docsstring
        with the wrapped function's doc string.
        """
    _doc = """:param default_dir: The default directory selected when the dialog spawns 
    :param default_file: The default filename used in the dialog
    """
    f.__doc__ = (f.__doc__ or '') + _doc
    return f



def LayoutOptions(label_color=None,
                  label_bg_color=None,
                  help_color=None,
                  help_bg_color=None,
                  error_color=None,
                  error_bg_color=None,
                  show_label=True,
                  show_help=True,
                  visible=True,
                  full_width=False):
    """
    Layout Options:
    ---------------

    Color options can be passed either as a hex string ('#ff0000') or as
    a collection of RGB values (e.g. `[255, 0, 0]` or `(255, 0, 0)`)

    :param label_color:    The foreground color of the label text
    :param label_bg_color: The background color of the label text.
    :param help_color:     The foreground color of the help text.
    :param help_bg_color:  The background color of the help text.
    :param error_color:    The foreground color of the error text (when visible).
    :param error_bg_color: The background color of the error text (when visible).
    :param show_label:     Toggles whether or not to display the label text
    :param show_help:      Toggles whether or not to display the help text
    :param visible:        Hides the entire widget when False. Note: the widget
                           is still present in the UI and will still send along any
                           default values that have been provided in code. This option
                           is here for when you want to hide certain advanced / dangerous
                           inputs from your GUI users.
    :param full_width:     This is a layout hint for this widget. When True the widget
                           will fill the entire available space within a given row.
                           Otherwise, it will be sized based on the column rules
                           provided elsewhere.
    """
    return _clean(locals())


@_include_layout_docs
def TextField(validator=None, **layout_options):
    return _clean(locals())


@_include_layout_docs
def PasswordField(validator=None, **layout_options):
    return _clean(locals())


@_include_layout_docs
def IntegerField(validator=None, min=0, max=100, increment=1, **layout_options):
    """
    :param min: The minimum value allowed
    :param max: The maximum value allowed
    :param increment: The step size of the spinner
    """
    return _clean(locals())

@_include_layout_docs
def Slider(validator=None, min=0, max=100, increment=1, **layout_options):
    """
    :param min: The minimum value allowed
    :param max: The maximum value allowed
    :param increment: The step size of the slider
    """
    return _clean(locals())


@_include_layout_docs
def DecimalField(validator=None,
                 min=0.0,
                 max=1.0,
                 increment=0.01,
                 precision=2,
                 **layout_options):
    """
    :param min: The minimum value allowed
    :param max: The maximum value allowed
    :param increment: The step size of the spinner
    :param precision: The precision of the decimal (0-20)
    """
    return _clean(locals())


@_include_layout_docs
def TextArea(height=None, readonly=False, validator=None, **layout_options):
    """
    :param height:   The height of the TextArea.
    :param readonly: Controls whether or not user's may modify the contents
    """
    return _clean(locals())


@_include_layout_docs
def RichTextConsole(**layout_options):
    return _clean(locals())


@_include_layout_docs
def ListBox(height=None, **layout_options):
    """
    :param height: The height of the ListBox
    """
    return _clean(locals())

# TODO: what are this guy's layout options..?
def MutexGroup(initial_selection=None, title=None, **layout_options):
    """
    :param initial_selection: The index of the option which should be initially selected.
    :param title:             Adds the supplied title above the RadioGroup options (when present)
    """
    return _clean(locals())


@_include_layout_docs
def Dropdown(**layout_options):
    return _clean(locals())


@_include_layout_docs
def Counter(**layout_options):
    return _clean(locals())


@_include_layout_docs
def CheckBox(**layout_options):
    return _clean(locals())


@_include_layout_docs
def BlockCheckBox(checkbox_label=None, **layout_options):
    return _clean(locals())


@_include_layout_docs
def FilterableDropdown(placeholder=None,
                       empty_message=None,
                       max_size=80,
                       search_strategy=None,
                       **layout_options):
    """
    :param placeholder:     Text to display when the user has provided no input
    :param empty_message:   Text to display if the user's query doesn't match anything
    :param max_size:        maximum height of the dropdown
    :param search_strategy: see: PrefixSearchStrategy
    """
    return _clean(locals())


def PrefixSearchStrategy(
                   choice_tokenizer=PrefixTokenizers.WORDS,
                   input_tokenizer=PrefixTokenizers.REGEX('\s'),
                   ignore_case=True,
                   operator='AND',
                   index_suffix=False):
    """
    :param choice_tokenizer: See: PrefixTokenizers - sets the tokenization strategy
                             for the `choices`
    :param input_tokenizer:  See: PrefixTokenizers sets how the users's `input` get tokenized.
    :param ignore_case:      Controls whether or not to honor case while searching
    :param operator:         see: `OperatorType` - controls whether or not individual
                             search tokens
                             get `AND`ed or `OR`d together when evaluating a match.
    :param index_suffix:     When enabled, generates a suffix-tree to enable efficient
                             partial-matching against any of the tokens.
    """
    return {**_clean(locals()), 'type': 'PrefixFilter'}


@_include_layout_docs
@_include_choose_dir_file_docs
@_include_chooser_msg_wildcard_docs
def FileChooser(wildcard=None,
                default_dir=None,
                default_file=None,
                message=None,
                **layout_options):
    return _clean(locals())


@_include_layout_docs
@_include_chooser_msg_wildcard_docs
def DirectoryChooser(wildcard=None,
                    default_path=None,
                    message=None,
                    **layout_options):
    """
    :param default_path: The default path selected when the dialog spawns
    """
    return _clean(locals())


@_include_layout_docs
@_include_choose_dir_file_docs
@_include_chooser_msg_wildcard_docs
def FileSaver(wildcard=None,
              default_dir=None,
              default_file=None,
              message=None,
              **layout_options):
    return _clean(locals())


@_include_layout_docs
@_include_choose_dir_file_docs
@_include_chooser_msg_wildcard_docs
def MultiFileSaver(wildcard=None,
              default_dir=None,
              default_file=None,
              message=None,
              **layout_options):
    return _clean(locals())


def ExpressionValidator(test=None, message=None):
    """
    Creates the data for a basic expression validator.

    Your test function can be made up of any valid Python expression.
    It receives the variable user_input as an argument against which to
    perform its validation. Note that all values coming from Gooey
    are in the form of a string, so you'll have to cast as needed
    in order to perform your validation.
    """
    return {**_clean(locals()), 'type': 'ExpressionValidator'}


def RegexValidator(test=None, message=None):
    """
    Creates the data for a basic RegexValidator.

    :param test:    the regex expression. This should be the expression
                    directly (i.e. `test='\d+'`). Gooey will test
                    that the user's input satisfies this expression.
    :param message: The message to display if the input doesn't match
                    the regex
    """
    return {**_clean(locals()), 'type': 'RegexValidator'}


def ArgumentGroup(show_border=False,
                  show_underline=True,
                  label_color=None,
                  columns=None,
                  margin_top=None):
    """
    :param show_border:    When True a labeled border will surround all widgets added to this group.
    :param show_underline: Controls whether or not to display the underline when using the default border style
    :param label_color:    The foreground color for the group name
    :param columns:        Controls the number of widgets on each row
    :param margin_top:     specifies the top margin in pixels for this group
    """
    return _clean(locals())





def _clean(options):
    cleaned = {k: v for k, v in options.items()
               if v is not None and k is not "layout_options"}
    return {**options.get('layout_options', {}), **cleaned}




import wx
import warnings

wx.Log.SetLogLevel(wx.LOG_Debug)

app = wx.App()

frame = wx.Frame(None, title='Simple application')
sizer = wx.BoxSizer(wx.HORIZONTAL)
sizer.Add(wx.StaticText(frame, -1, "foo"), 1, wx.ALIGN_CENTER_HORIZONTAL)

frame.Show()
app.MainLoop()


import wx
from wx.lib.scrolledpanel import ScrolledPanel

from gooey.gui.components.util.wrapped_static_text import AutoWrappedStaticText
from gooey.gui.util import wx_util
from gooey.util.functional import getin, flatmap, compact, indexunique
from gooey.gui.lang.i18n import _
from gooey.gui.components.mouse import notifyMouseEvent


class ConfigPage(ScrolledPanel):
    def __init__(self, parent, rawWidgets, buildSpec,  *args, **kwargs):
        super(ConfigPage, self).__init__(parent, *args, **kwargs)
        self.SetupScrolling(scroll_x=False, scrollToTop=False)
        self.rawWidgets = rawWidgets
        self.buildSpec = buildSpec
        self.reifiedWidgets = []
        self.layoutComponent()
        self.Layout()
        self.widgetsMap = indexunique(lambda x: x._id, self.reifiedWidgets)
        self.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
        ## TODO: need to rethink what uniquely identifies an argument.
        ## Out-of-band IDs, while simple, make talking to the client program difficult
        ## unless they're agreed upon before hand. Commands, as used here, have the problem
        ## of (a) not being nearly granular enough (for instance,  `-v` could represent totally different
        ## things given context/parser position), and (b) cannot identify positional args.

    def getName(self, group):
        """
        retrieve the group name from the group object while accounting for
        legacy fixed-name manual translation requirements.
        """
        name = group['name']
        return (_(name)
                if name in {'optional_args_msg', 'required_args_msg'}
                else name)


    def firstCommandIfPresent(self, widget):
        commands = widget._meta['commands']
        return commands[0] if commands else ''

    def getPositionalArgs(self):
        return [widget.getValue()['cmd'] for widget in self.reifiedWidgets
                if widget.info['cli_type'] == 'positional']

    def getOptionalArgs(self):
        return [widget.getValue()['cmd'] for widget in self.reifiedWidgets
                if widget.info['cli_type'] != 'positional']


    def isValid(self):
        states = [widget.getValue() for widget in self.reifiedWidgets]
        return not any(compact([state['error'] for state in states]))


    def seedUI(self, seeds):
        radioWidgets = self.indexInternalRadioGroupWidgets()
        for id, values in seeds.items():
            if id in self.widgetsMap:
                self.widgetsMap[id].setOptions(values)
            if id in radioWidgets:
                radioWidgets[id].setOptions(values)

    def indexInternalRadioGroupWidgets(self):
        groups = filter(lambda x: x.info['type'] == 'RadioGroup', self.reifiedWidgets)
        widgets = flatmap(lambda group: group.widgets, groups)
        return indexunique(lambda x: x._id, widgets)


    def displayErrors(self):
        states = [widget.getValue() for widget in self.reifiedWidgets]
        errors = [state for state in states if state['error']]
        for error in errors:
            widget = self.widgetsMap[error['id']]
            widget.setErrorString(error['error'])
            widget.showErrorString(True)
            while widget.GetParent():
                widget.Layout()
                widget = widget.GetParent()

    def resetErrors(self):
        for widget in self.reifiedWidgets:
            widget.setErrorString('')
            widget.showErrorString(False)

    def hideErrors(self):
        for widget in self.reifiedWidgets:
            widget.hideErrorString()


    def layoutComponent(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        for item in self.rawWidgets['contents']:
            self.makeGroup(self, sizer, item, 0, wx.EXPAND | wx.ALL, 10)
        self.SetSizer(sizer)

    def makeGroup(self, parent, thissizer, group, *args):
        '''
        Messily builds the (potentially) nested and grouped layout

        Note! Mutates `self.reifiedWidgets` in place with the widgets as they're
        instantiated! I cannot figure out how to split out the creation of the
        widgets from their styling without WxPython violently exploding

        TODO: sort out the WX quirks and clean this up.
        '''

        # determine the type of border , if any, the main sizer will use
        if getin(group, ['options', 'show_border'], False):
            boxDetails = wx.StaticBox(parent, -1, self.getName(group) or '')
            boxSizer = wx.StaticBoxSizer(boxDetails, wx.VERTICAL)
        else:
            boxSizer = wx.BoxSizer(wx.VERTICAL)
            boxSizer.AddSpacer(10)
            if group['name']:
                groupName = wx_util.h1(parent, self.getName(group) or '')
                groupName.SetForegroundColour(getin(group, ['options', 'label_color']))
                groupName.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
                boxSizer.Add(groupName, 0, wx.TOP | wx.BOTTOM | wx.LEFT, 8)

        group_description = getin(group, ['description'])
        if group_description:
            description = AutoWrappedStaticText(parent, label=group_description, target=boxSizer)
            description.SetForegroundColour(getin(group, ['options', 'description_color']))
            description.SetMinSize((0, -1))
            description.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
            boxSizer.Add(description, 1,  wx.EXPAND | wx.LEFT | wx.RIGHT | wx.BOTTOM, 10)

        # apply an underline when a grouping border is not specified
        # unless the user specifically requests not to show it
        if not getin(group, ['options', 'show_border'], False) and group['name'] \
                and getin(group, ['options', 'show_underline'], True):
            boxSizer.Add(wx_util.horizontal_rule(parent), 0, wx.EXPAND | wx.LEFT, 10)

        ui_groups = self.chunkWidgets(group)

        for uigroup in ui_groups:
            sizer = wx.BoxSizer(wx.HORIZONTAL)
            for item in uigroup:
                widget = self.reifyWidget(parent, item)
                if not getin(item, ['options', 'visible'], True):
                    widget.Hide()
                # !Mutate the reifiedWidgets instance variable in place
                self.reifiedWidgets.append(widget)
                sizer.Add(widget, 1, wx.ALL | wx.EXPAND, 5)
            boxSizer.Add(sizer, 0, wx.ALL | wx.EXPAND, 5)

        # apply the same layout rules recursively for subgroups
        hs = wx.BoxSizer(wx.HORIZONTAL)
        for e, subgroup in enumerate(group['groups']):
            self.makeGroup(parent, hs, subgroup, 1, wx.EXPAND)
            if len(group['groups']) != e:
                hs.AddSpacer(5)

            # self.makeGroup(parent, hs, subgroup, 1, wx.ALL | wx.EXPAND, 5)
            itemsPerColumn = getin(group, ['options', 'columns'], 2)
            if e % itemsPerColumn or (e + 1) == len(group['groups']):
                boxSizer.Add(hs, *args)
                hs = wx.BoxSizer(wx.HORIZONTAL)


        group_top_margin = getin(group, ['options', 'margin_top'], 1)

        marginSizer = wx.BoxSizer(wx.VERTICAL)
        marginSizer.Add(boxSizer, 1, wx.EXPAND | wx.TOP, group_top_margin)

        thissizer.Add(marginSizer, *args)


    def chunkWidgets(self, group):
        ''' chunk the widgets up into groups based on their sizing hints '''
        ui_groups = []
        subgroup = []
        for index, item in enumerate(group['items']):
            if getin(item, ['options', 'full_width'], False):
                ui_groups.append(subgroup)
                ui_groups.append([item])
                subgroup = []
            else:
                subgroup.append(item)
            if len(subgroup) == getin(group, ['options', 'columns'], 2) \
                    or item == group['items'][-1]:
                ui_groups.append(subgroup)
                subgroup = []
        return ui_groups


    def reifyWidget(self, parent, item):
        ''' Convert a JSON description of a widget into a WxObject '''
        from gooey.gui.components import widgets
        widgetClass = getattr(widgets, item['type'])
        return widgetClass(parent, item)



class TabbedConfigPage(ConfigPage):
    """
    Splits top-level groups across tabs
    """


    def layoutComponent(self):
        # self.rawWidgets['contents'] = self.rawWidgets['contents'][1:2]
        self.notebook = wx.Notebook(self, style=wx.BK_DEFAULT)

        panels = [wx.Panel(self.notebook) for _ in self.rawWidgets['contents']]
        sizers = [wx.BoxSizer(wx.VERTICAL) for _ in panels]

        for group, panel, sizer in zip(self.rawWidgets['contents'], panels, sizers):
            self.makeGroup(panel, sizer, group, 0, wx.EXPAND)
            panel.SetSizer(sizer)
            panel.Layout()
            self.notebook.AddPage(panel, group['name'])
            self.notebook.Layout()


        _sizer = wx.BoxSizer(wx.VERTICAL)
        _sizer.Add(self.notebook, 1, wx.EXPAND)
        self.SetSizer(_sizer)
        self.Layout()







"""
Util for talking to the client program in order to retrieve
dynamic defaults for the UI
"""
import json
import subprocess


def fetchDynamicProperties(target, encoding):
    """
    Sends a gooey-seed-ui request to the client program it retrieve
    dynamically generated defaults with which to seed the UI
    """
    # TODO: this needs to apply the same argpase_to_json data cleaning rules
    cmd = '{} {}'.format(target, 'gooey-seed-ui --ignore-gooey')
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    if proc.returncode != 0:
        out, _ = proc.communicate()
        return json.loads(out.decode(encoding))
    else:
        # TODO: useful feedback
        return {}




from gooey import Gooey, GooeyParser

@Gooey(target="ffmpeg", program_name='Frame Extraction v1.0', suppress_gooey_flag=True)
def main():
    parser = GooeyParser(description="Extracting frames from a movie using FFMPEG")
    ffmpeg = parser.add_argument_group('Frame Extraction Util')
    ffmpeg.add_argument('-i',
                        metavar='Input Movie',
                        help='The movie for which you want to extract frames',
                        required=True,  # Look at me!
                        widget='FileChooser')
    ffmpeg.add_argument('output',
                        metavar='Output Image',
                        help='Where to save the extracted frame',
                        widget='FileSaver',
                        )
    ffmpeg.add_argument('-ss',
                        metavar='Timestamp',
                        required=True,  # Look at me!
                        help='Timestamp of snapshot (in seconds)',
                        gooey_options={
                            'validator': {
                                'test': 'user_input.isdigit()',
                                'message': 'Please enter a number'
                            }
                        })
    ffmpeg.add_argument('-frames:v',
                        metavar='Timestamp',
                        default=1,
                        required=True,  # Look at me!
                        gooey_options={
                            'visible': False,
                        })

    parser.parse_args()
    print('Done.')

if __name__ == '__main__':
    main()



'''
A Gooey example showing use of richtext display in the console pannel.
'''

from gooey import Gooey, GooeyParser
from colored import stylize, attr, fg

@Gooey(auto_start=True)
def main():
    parser = GooeyParser(description='Just display the console')

    args = parser.parse_args()
    print(stylize("This is bold.", attr("bold")))
    print(stylize("This is underlined.", attr("underlined")))
    print(stylize("This is green.", fg("green")))
    print(stylize("This is red.", fg("red")))
    print(stylize("This is blue and bold.", fg("blue") + attr("bold")))


if __name__ == '__main__':
    main()


import argparse
import sys
import unittest
from argparse import ArgumentParser

from gooey import GooeyParser
from gooey.python_bindings import argparse_to_json
from gooey.util.functional import getin
from gooey.tests import *

class TestArgparse(unittest.TestCase):

    def test_mutex_groups_conversion(self):
        """
        Ensure multiple mutex groups are processed correctly.
        """
        parser = ArgumentParser()
        g1 = parser.add_mutually_exclusive_group(required=True)
        g1.add_argument('--choose1')
        g1.add_argument('--choose2')

        g2 = parser.add_mutually_exclusive_group(required=True)
        g2.add_argument('--choose3')
        g2.add_argument('--choose4')

        output = argparse_to_json.process(parser, {}, {}, {})

        # assert that we get two groups of two choices back
        items = output[0]['items']
        self.assertTrue(len(items) == 2)
        group1 = items[0]
        group2 = items[1]
        self.assertTrue(['--choose1'] in group1['data']['commands'])
        self.assertTrue(['--choose2'] in group1['data']['commands'])
        self.assertTrue(['--choose3'] in group2['data']['commands'])
        self.assertTrue(['--choose4'] in group2['data']['commands'])
        self.assertTrue(group1['type'] == 'RadioGroup')
        self.assertTrue(group2['type'] == 'RadioGroup')

    def test_json_iterable_conversion(self):
        """
        Issue #312 - tuples weren't being coerced to list during argparse
        conversion causing downstream issues when concatenating
        """
        # our original functionality accepted only lists as the choices arg
        parser = ArgumentParser()
        parser.add_argument("-foo", choices=['foo','bar', 'baz'])
        result = argparse_to_json.action_to_json(parser._actions[-1], "Dropdown", {})

        choices = result['data']['choices']
        self.assertTrue(isinstance(choices, list))
        self.assertEqual(choices, ['foo','bar', 'baz'])

        # Now we allow tuples as well.
        parser = ArgumentParser()
        parser.add_argument("-foo", choices=('foo','bar', 'baz'))
        result = argparse_to_json.action_to_json(parser._actions[-1], "Dropdown", {})

        choices = result['data']['choices']
        self.assertTrue(isinstance(choices, list))
        self.assertEqual(choices, ['foo','bar', 'baz'])


    def test_choice_string_cooersion(self):
        """
        Issue 321 - must coerce choice types to string to support wx.ComboBox
        """
        parser = ArgumentParser()
        parser.add_argument('--foo', default=1, choices=[1, 2, 3])
        choice_action = parser._actions[-1]
        result = argparse_to_json.action_to_json(choice_action, 'Dropdown', {})
        self.assertEqual(getin(result, ['data', 'choices']), ['1', '2', '3'])
        # default value is also converted to a string type
        self.assertEqual(getin(result, ['data', 'default']), '1')

    def test_choice_string_cooersion_no_default(self):
        """
        Make sure that choice types without a default don't create
        the literal string "None" but stick with the value None
        """
        parser = ArgumentParser()
        parser.add_argument('--foo', choices=[1, 2, 3])

        choice_action = parser._actions[-1]
        result = argparse_to_json.action_to_json(choice_action, 'Dropdown', {})
        self.assertEqual(getin(result, ['data', 'default']), None)
        

    def test_listbox_defaults_cast_correctly(self):
        """
        Issue XXX - defaults supplied in a list were turned into a string
        wholesale (list and all). The defaults should be stored as a list
        proper with only the _internal_ values coerced to strings.
        """
        parser = GooeyParser()
        parser.add_argument('--foo', widget="Listbox", nargs="*", choices=[1, 2, 3], default=[1, 2])

        choice_action = parser._actions[-1]
        result = argparse_to_json.action_to_json(choice_action, 'Listbox', {})
        self.assertEqual(getin(result, ['data', 'default']), ['1', '2'])


    def test_listbox_single_default_cast_correctly(self):
        """
        Single arg defaults to listbox should be wrapped in a list and
        their contents coerced as usual.
        """
        parser = GooeyParser()
        parser.add_argument('--foo', widget="Listbox",
                            nargs="*", choices=[1, 2, 3], default="sup")

        choice_action = parser._actions[-1]
        result = argparse_to_json.action_to_json(choice_action, 'Listbox', {})
        self.assertEqual(getin(result, ['data', 'default']), ['sup'])

    def test_non_data_defaults_are_dropped_entirely(self):
        """
        This is a refinement in understanding of Issue #147

        Caused by Issue 377 - passing arbitrary objects as defaults
        causes failures.
        """
        # passing plain data to cleaning function results in plain data
        # being returned
        data = ['abc',
                123,
                ['a', 'b'],
                [1, 2, 3]]

        for datum in data:
            result = argparse_to_json.clean_default(datum)
            self.assertEqual(result, datum)

        # passing in complex objects results in None
        objects = [sys.stdout, sys.stdin, object(), max, min]

        for obj in objects:
            result = argparse_to_json.clean_default(obj)
            self.assertEqual(result, None)


    def test_suppress_is_removed_as_default_value(self):
        """
        Issue #469
        Argparse uses the literal string ==SUPPRESS== as an internal flag.
        When encountered in Gooey, these should be dropped and mapped to `None`.
        """
        parser = ArgumentParser(prog='test_program')
        parser.add_argument("--foo", default=argparse.SUPPRESS)
        parser.add_argument('--version', action='version', version='1.0')

        result = argparse_to_json.convert(parser, num_required_cols=2, num_optional_cols=2)
        groups = getin(result, ['widgets', 'test_program', 'contents'])
        for item in groups[0]['items']:
            self.assertEqual(getin(item, ['data', 'default']), None)


    def test_textinput_with_list_default_mapped_to_cli_friendly_value(self):
        """
        Issue: #500

        Using nargs and a `default` value with a list causes the literal list string
        to be put into the UI.
        """
        testcases = [
            {'nargs': '+', 'default': ['a b', 'c'], 'gooey_default': '"a b" "c"', 'w': 'TextField'},
            {'nargs': '*', 'default': ['a b', 'c'], 'gooey_default': '"a b" "c"', 'w': 'TextField'},
            {'nargs': '...', 'default': ['a b', 'c'], 'gooey_default': '"a b" "c"', 'w': 'TextField'},
            {'nargs': 2, 'default': ['a b', 'c'], 'gooey_default': '"a b" "c"', 'w': 'TextField'},
            # TODO: this demos the current nargs behavior for string defaults, but
            # TODO: it is wrong! These should be wrapped in quotes so spaces aren't
            # TODO: interpreted as unique arguments.
            {'nargs': '+', 'default': 'a b', 'gooey_default': 'a b', 'w': 'TextField'},
            {'nargs': '*', 'default': 'a b', 'gooey_default': 'a b', 'w': 'TextField'},
            {'nargs': '...', 'default': 'a b', 'gooey_default': 'a b', 'w': 'TextField'},
            {'nargs': 1, 'default': 'a b', 'gooey_default': 'a b', 'w': 'TextField'},

            # Listbox has special nargs handling which keeps the list in tact.
            {'nargs': '+', 'default': ['a b', 'c'], 'gooey_default': ['a b', 'c'], 'w': 'Listbox'},
            {'nargs': '*', 'default': ['a b', 'c'], 'gooey_default': ['a b', 'c'], 'w': 'Listbox'},
            {'nargs': '...', 'default': ['a b', 'c'], 'gooey_default': ['a b', 'c'],'w': 'Listbox'},
            {'nargs': 2, 'default': ['a b', 'c'], 'gooey_default': ['a b', 'c'], 'w': 'Listbox'},
            {'nargs': '+', 'default': 'a b', 'gooey_default': ['a b'], 'w': 'Listbox'},
            {'nargs': '*', 'default': 'a b', 'gooey_default': ['a b'], 'w': 'Listbox'},
            {'nargs': '...', 'default': 'a b', 'gooey_default': ['a b'], 'w': 'Listbox'},
            {'nargs': 1, 'default': 'a b', 'gooey_default': ['a b'], 'w': 'Listbox'},
        ]
        for case in testcases:
            with self.subTest(case):
                parser = ArgumentParser(prog='test_program')
                parser.add_argument('--foo', nargs=case['nargs'], default=case['default'])
                action = parser._actions[-1]
                result = argparse_to_json.handle_default(action, case['w'])
                self.assertEqual(result, case['gooey_default'])

    def test_nargs(self):
        """
        so there are just a few simple rules here:
        if nargs in [*, N, +, remainder]:
            default MUST be a list OR we must map it to one

        action:_StoreAction
            - nargs '?'
                - default:validate list is invalid
                - default:coerce stringify
            - nargs #{*, N, +, REMAINDER}
                - default:validate None
                - default:coerce
                    if string: stringify
                    if list: convert from list to cli style input string
        action:_StoreConstAction
            - nargs: invalid
            - defaults:stringify

        action:{_StoreFalseAction, _StoreTrueAction}
            - nargs: invalid
            - defaults:validate: require bool
            - defaults:coerce: no stringify; leave bool

        action:_CountAction
            - nargs: invalid
            - default:validate: must be numeric index within range OR None
            - default:coerce: integer or None

        action:_AppendAction
            TODO: NOT CURRENTLY SUPPORTED BY GOOEY
            nargs behavior is weird and needs to be understood.
            - nargs

        action:CustomUserAction:
            - nargs: no way to know expected behavior. Ignore
            - default: jsonify type if possible.
        """

        parser = ArgumentParser()
        parser.add_argument(
            '--bar',
            nargs='+',
            choices=["one", "two"],
            default="one",
        )



import re
import itertools

r = r'\b((?:file|https?)://[^\s<>]*)'
t = 'hello world https://www.url-with-querystring.com/?url=has-querystring asdf asd sih \n\nhttp://www.example.com'


def grouper(iterable, n, fillvalue=None):
    "Collect data into fixed-length chunks or blocks"
    # grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx
    args = [iter(iterable)] * n
    return itertools.zip_longest(fillvalue=fillvalue, *args)

# print(list(grouper(re.finditer(r, t[:-26]), 2)))

# identity = lambda x: x

def iddd(*args):
    print("Uhh??")
    print(args)

# print(re.split(r, 'http://www.example.com'))
# print(re.split(r, 'Hello world!\n\nhttps://www.example.com'))
# print(re.split(r, 'file://www.example.com'))
# print(re.split(r, 'some text file://www.example.com some more text http://www.example.com http://www.example.com'))


def ddothing(x):
    textStream = iter(re.split(r, x))
    for item in textStream:
        plaintext = item
        url = next(textStream, None)
        print(repr(plaintext))
        if url:
            print(repr(url))
    # try:
    #     while textStream:
    #         plaintext = next(textStream)
    #         url = next(textStream, None)
    #
    #         print(repr(plaintext))
    #         if url:
    #             print(repr(url))
    # except StopIteration:
    #     pass

ddothing('http://ww.example.com\n\nhttps://asdfasdf.com')
print('==========================')
ddothing('http://ww.example.com')
print('==========================')
ddothing('HelloWorld http://ww.example.com\n\nhttps://asdfasdf.com')
print('==========================')
ddothing('')

#
#
# for plaintext, url in grouper(re.split(r, ''), 2):
#     self.WriteText(plaintext)
#     if url:
#         self.BeginTextColour(self.url_colour)
#         self.BeginUnderline()
#         self.BeginURL(url)
#         self.WriteText(url)
#         self.EndURL()
#         self.EndUnderline()
#         self.EndTextColour()





from __future__ import unicode_literals
from __future__ import print_function
import sys
from time import sleep
from gooey import Gooey, GooeyParser


@Gooey(progress_regex=r"^progress: (?P<current>\d+)/(?P<total>\d+)$",
       progress_expr="current / total * 100")
def main():
    parser = GooeyParser(prog="example_progress_bar_3")
    parser.add_argument("steps", type=int, default=15)
    parser.add_argument("delay", type=int, default=1)
    args = parser.parse_args(sys.argv[1:])

    for i in range(args.steps):
        print("progress: {}/{}".format(i+1, args.steps))
        sys.stdout.flush()
        sleep(args.delay)


if __name__ == "__main__":
    sys.exit(main())



import time
import unittest
from argparse import ArgumentParser
from itertools import *

from tests.harness import instrumentGooey


from gooey.tests import *

class TestFooterTimeRemaining(unittest.TestCase):

    def make_parser(self):
        parser = ArgumentParser(description='description')
        return parser

    def test_time_remaining_visibility(self):
        for testdata in self.testcases():
            with self.subTest(testdata):
                with instrumentGooey(self.make_parser(), timing_options=testdata) as (app, gooeyApp):

                    gooeyApp.showConsole()
                    footer = gooeyApp.footer

                    self.assertEqual(
                        footer.time_remaining_text.Shown,
                        testdata.get('show_time_remaining',False)
                    )

    def test_time_remaining_visibility_on_complete(self):
        for testdata in self.testcases():
            with self.subTest(testdata):
                with instrumentGooey(self.make_parser(), timing_options=testdata) as (app, gooeyApp):

                    gooeyApp.showComplete()
                    footer = gooeyApp.footer


                    if not testdata.get('show_time_remaining') and testdata:
                        self.assertEqual(
                            footer.time_remaining_text.Shown,
                            testdata.get('hide_time_remaining_on_complete',True)
                        )
                    else:
                        return True

    def testcases(self):
        """
        Generate a powerset of all possible combinations of
        the header parameters (empty, some present, all present, all combos)
        """
        iterable = product(['show_time_remaining', 'hide_time_remaining_on_complete'], [True, False])
        allCombinations = list(powerset(iterable))
        return [{k: v for k,v in args}
                for args in allCombinations]


def powerset(iterable):
    "powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)"
    s = list(iterable)
    return chain.from_iterable(combinations(s, r) for r in range(len(s)+1))


if __name__ == '__main__':
    unittest.main()


from gooey import Gooey
from gooey import GooeyParser


@Gooey(
    program_name='ICC-OBS Version 0.1',
    show_stop_warning=False,
    force_stop_is_error=False,
    disable_progress_bar_animation=True,
    required_cols=1,
    default_size=(1080, 1080),
    advanced=True,
    navigation='TABBED',
    # image_dir=resource_path('images'),
    menu=[{
        'name': 'File',
        'items': [{
            'type': 'AboutDialog',
            'menuTitle': 'About',
            'name': 'Gooey Layout Demo',
            'description': 'An example of Gooey\'s layout flexibility',
            'version': '1.2.1',
            'copyright': '2018',
            'website': 'https://github.com/chriskiehl/Gooey',
            'developer': 'http://chriskiehl.com/',
            'license': 'MIT'
        }, {
            'type': 'MessageDialog',
            'menuTitle': 'Information',
            'caption': 'My Message',
            'message': 'I am demoing an informational dialog!'
        }, {
            'type': 'Link',
            'menuTitle': 'Get Data',
            'url': 'https://data.ccca.ac.at/group/climaproof'
        }]
    }, {
        'name': 'Help',
        'items': [{
            'type': 'Link',
            'menuTitle': 'Documentation',
            'url': 'https://www.readthedocs.com/foo'
        }]
    }]
)
def main():
    parser = GooeyParser(description='Example About Dialoge')
    parser.add_argument('filename', help="name of the file to process")
    args = parser.parse_args()


if __name__ == "__main__":
    main()



import time

from gooey import Gooey
from gooey import GooeyParser


@Gooey(
    sidebar_title="Your Custom Title",
    show_sidebar=True,
    show_success_modal=False,
    force_stop_is_error=False,
)
def main():
    parser = get_parser()
    args = parser.parse_args()
    time.sleep(2)
    print("Success")


def get_parser():
    """
    A simple parser with a single required argument and no default thus
    ensuring that clicking the start button in the UI will throw
    a validation error.
    """
    desc = "Example application to show Gooey's various widgets"
    parser = GooeyParser(description=desc, add_help=False)
    parser.add_argument('--textfield', widget="TextField", required=True)
    return parser


if __name__ == '__main__':
    main()


import unittest

from tests.harness import instrumentGooey
from gooey import GooeyParser
from gooey.tests import *

class TestPasswordField(unittest.TestCase):

    def makeParser(self, **kwargs):
        parser = GooeyParser(description='description')
        parser.add_argument('--widget', widget="PasswordField", **kwargs)
        return parser


    def testPlaceholder(self):
        cases = [
            [{}, ''],
            [{'placeholder': 'Hello'}, 'Hello']
        ]
        for options, expected in cases:
            parser = self.makeParser(gooey_options=options)
            with instrumentGooey(parser) as (app, gooeyApp):
                # because of how poorly designed the Gooey widgets are
                # we have to reach down 3 levels in order to find the
                # actual WX object we need to test.
                widget = gooeyApp.configs[0].reifiedWidgets[0].widget
                self.assertEqual(widget.widget.GetHint(), expected)



if __name__ == '__main__':
    unittest.main()


from contextlib import contextmanager

import wx
import wx.html

import gooey.gui.events as events
from gooey.gui.components.filtering.prefix_filter import PrefixSearch
from gooey.gui.components.mouse import notifyMouseEvent
from gooey.gui.components.widgets.dropdown import Dropdown
from gooey.gui.lang.i18n import _
from gooey.gui.pubsub import pub

__ALL__ = ('FilterableDropdown',)


class FilterableDropdown(Dropdown):
    """
    TODO: tests for gooey_options
    TODO: documentation
    A dropdown with auto-complete / filtering behaviors.

    This is largely a recreation of the `AutoComplete` functionality baked
    into WX itself.

    Background info:
    The Dropdown's listbox and its Autocomplete dialog are different components.
    This means that if the former is open, the latter cannot be used. Additionally,
    this leads to annoying UX quirks like the boxes having different styles and sizes.
    If we ignore the UX issues, a possible solution for still leveraging the current built-in
    AutoComplete functionality would have been to capture EVT_TEXT and conditionally
    close the dropdown while spawning the AutoComplete dialog, but due to
    (a) non-overridable behaviors and (b) lack a fine grained events, this cannot be
    done in a seamless manner.

    FAQ:
    Q: Why does it slide down rather than hover over elements like the native ComboBox?
    A: The only mecahnism for layering in WX is the wx.PopupTransientWindow. There's a long
       standing issue in wxPython which prevents ListBox/Ctrl from capturing events when
       inside of a PopupTransientWindow (see: https://tinyurl.com/y28ngh7v)

    Q: Why is visibility handled by changing its size rather than using Show/Hide?
    A: WX's Layout engine is strangely quirky when it comes to toggling visibility.
       Repeated calls to Layout() after the first show/hide cycle no longer produce
       the same results. I have no idea why. I keep checking it thinking I'm crazy, but
       alas... seems to be the case.
    """
    gooey_options = {
        'placeholder': str,
        'empty_message': str,
        'max_size': str
    }
    def __init__(self, *args, **kwargs):
        # these are declared here and created inside
        # of getWidget() because the structure of all
        # this inheritance garbage is broken.
        self.listbox = None
        self.model = None
        super(FilterableDropdown, self).__init__(*args, **kwargs)
        self.SetDoubleBuffered(True)

    def interpretState(self, model):
        """
        Updates the UI to reflect the current state of the model.
        """
        if self.widget.GetValue() != self.model.displayValue:
            self.widget.ChangeValue(model.displayValue)

        self.listbox.Clear()
        self.listbox.SetItemCount(len(self.model.suggestions))
        if len(self.model.suggestions) == 1:
            # I have no clue why this is required, but without
            # manually flicking the virtualized listbox off/on
            # it won't paint the update when there's only a single
            # item being displayed
            self.listbox.Show(False)
            self.listbox.Show(self.model.suggestionsVisible)
        if model.selectedSuggestion > -1:
            self.listbox.SetSelection(model.selectedSuggestion)
            self.widget.SetInsertionPoint(-1)
            self.widget.SetSelection(999, -1)
        else:
            self.listbox.SetSelection(-1)
        self.estimateBestSize()
        self.listbox.Show(self.model.suggestionsVisible)
        self.Layout()
        self.GetParent().Layout()

    def onComponentInitialized(self):
        self.widget.GetTextCtrl().Bind(wx.EVT_TEXT, self.onTextInput)
        self.widget.GetTextCtrl().Bind(wx.EVT_CHAR_HOOK, self.onKeyboardControls)
        self.widget.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
        self.widget.GetTextCtrl().Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
        self.listbox.Bind(wx.EVT_LISTBOX, self.onClickSuggestion)
        pub.subscribe(events.LEFT_DOWN, self.onMouseClick)
        self.widget.SetHint(self._options.get('placeholder', ''))

    def getWidget(self, parent, *args, **options):
        # self.widget = wx.ComboCtrl(parent)
        self.comboCtrl = wx.ComboCtrl(parent)
        self.comboCtrl.OnButtonClick = self.onButton
        self.foo = ListCtrlComboPopup()
        self.comboCtrl.SetPopupControl(self.foo)
        self.listbox = VirtualizedListBox(self)
        self.listbox.OnGetItem = self.OnGetItem
        # model is created here because the design of these widget
        # classes is broken.
        self.model = FilterableDropdownModel(self._meta['choices'], self._options, listeners=[self.interpretState])
        # overriding this to false removes it from tab behavior.
        # and keeps the tabbing at the top-level widget level
        self.listbox.AcceptsFocusFromKeyboard = lambda *args, **kwargs: False
        return self.comboCtrl

    def OnGetItem(self, n):
        return self.model.suggestions[n]

    def getSublayout(self, *args, **kwargs):
        verticalSizer = wx.BoxSizer(wx.VERTICAL)
        layout = wx.BoxSizer(wx.HORIZONTAL)
        layout.Add(self.widget, 1, wx.EXPAND)
        verticalSizer.Add(layout, 0, wx.EXPAND)
        verticalSizer.Add(self.listbox, 0, wx.EXPAND)
        self.listbox.SetMaxSize(self.model.maxSize)
        self.listbox.Hide()
        self.Layout()
        return verticalSizer

    def setOptions(self, options):
        self.model.updateChoices(options)
        if not self.model.actualValue in options:
            self.model.updateActualValue('')

    def setValue(self, value):
        self.model.updateActualValue(value)

    def onButton(self):
        if self.model.suggestionsVisible:
            self.model.hideSuggestions()
        else:
            self.model.showSuggestions()

    def onClickSuggestion(self, event):
        self.model.acceptSuggestion(self.model.suggestions[event.Selection])
        event.Skip()

    def onMouseClick(self, wxEvent):
        """
        Closes the suggestions when the user clicks anywhere
        outside of the current widget.
        """
        if wxEvent.EventObject not in (self.widget, self.widget.GetTextCtrl()):
            self.model.hideSuggestions()
            wxEvent.Skip()
        else:
            wxEvent.Skip()

    def onTextInput(self, event):
        """Processes the user's input and show relevant suggestions"""
        self.model.handleTextInput(event.GetString())

    def onKeyboardControls(self, event):
        """
        Handles any keyboard events relevant to the
        control/navigation of the suggestion box.
        All other events are passed through via `Skip()`
        and bubble up to `onTextInput` to be handled.
        """
        if event.GetKeyCode() == wx.WXK_ESCAPE:
            self.model.ignoreSuggestions()
        elif event.GetKeyCode() in (wx.WXK_TAB, wx.WXK_RETURN):
            self.model.acceptSuggestion(self.model.displayValue)
            event.Skip()
        elif event.GetKeyCode() in (wx.WXK_DOWN, wx.WXK_UP):
            if not self.model.suggestionsVisible:
                self.model.generateSuggestions(self.model.displayValue)
                self.model.showSuggestions()
            else:
                if self.listbox.OnGetItem(0) != self.model.noMatch:
                    self.ignore = True
                    if event.GetKeyCode() == wx.WXK_DOWN:
                        self.model.incSelectedSuggestion()
                    else:
                        self.model.decSelectedSuggestion()
        else:
            # for some reason deleting text doesn't
            # trigger the usual evt_text event, even though
            # it IS a modification of the text... so handled here.
            if event.GetKeyCode() == wx.WXK_DELETE:
                self.model.handleTextInput('')
            event.Skip()

    def estimateBestSize(self):
        """
        Restricts the size of the dropdown based on the number
        of items within it. This is a rough estimate based on the
        current font size.
        """
        padding = 11
        rowHeight = self.listbox.GetFont().GetPixelSize()[1] + padding
        maxHeight = self.model.maxSize[1]
        self.listbox.SetMaxSize((-1, min(maxHeight, len(self.model.suggestions) * rowHeight)))
        self.listbox.SetMinSize((-1, min(maxHeight, len(self.model.suggestions) * rowHeight)))
        self.listbox.SetSize((-1, -1))



class VirtualizedListBox(wx.html.HtmlListBox):
    def __init__(self, *args, **kwargs):
        super(VirtualizedListBox, self).__init__(*args, **kwargs)
        self.SetItemCount(1)

    def OnGetItem(self, n):
        return ''




class FilterableDropdownModel(object):
    """
    The model/state for the FilterableDropdown. While this is still one
    big ball of mutation (hard to get away from in WX), it serves the purpose
    of keeping data transforms independent of presentation concerns.
    """
    gooey_options = {
        'placeholder': str,
        'empty_message': str,
        'max_size': str
    }
    def __init__(self, choices, options, listeners=[], *args, **kwargs):
        self.listeners = listeners
        self.actualValue = ''
        self.displayValue = ''
        self.dropEvent = False
        self.suggestionsVisible = False
        self.noMatch = options.get('no_matches', _('dropdown.no_matches'))
        self.choices = choices
        self.suggestions = choices
        self.selectedSuggestion = -1
        self.suggestionsVisible = False
        self.maxSize = (-1, options.get('max_size', 80))
        self.strat = PrefixSearch(choices, options.get('search_strategy', {}))

    def __str__(self):
        return str(vars(self))

    @contextmanager
    def notify(self):
        try:
            yield
        finally:
            for listener in self.listeners:
                listener(self)

    def updateChoices(self, choices):
        """Update the available choices in response
        to a dynamic update"""
        self.choices = choices
        self.strat.updateChoices(choices)

    def handleTextInput(self, value):
        if self.dropEvent:
            self.dropEvent = False
        else:
            with self.notify():
                self.actualValue = value
                self.displayValue = value
                self.selectedSuggestion = -1
                self.generateSuggestions(value)
                self.suggestionsVisible = True

    def updateActualValue(self, value):
        with self.notify():
            self.actualValue = value
            self.displayValue = value

    def acceptSuggestion(self, suggestion):
        """Accept the currently selected option as the user's input"""
        with self.notify():
            self.actualValue = suggestion
            self.displayValue = suggestion
            self.suggestionsVisible = False
            self.selectedSuggestion = -1

    def ignoreSuggestions(self):
        """
        Ignore the suggested values and replace the
        user's original input.
        """
        with self.notify():
            self.displayValue = self.actualValue
            self.suggestionsVisible = False
            self.selectedSuggestion = -1

    def generateSuggestions(self, prompt):
        suggestions = self.strat.findMatches(prompt)
        final_suggestions = suggestions if suggestions else [self.noMatch]
        self.suggestions = final_suggestions

    def incSelectedSuggestion(self):
        with self.notify():
            nextIndex = (self.selectedSuggestion + 1) % len(self.suggestions)
            suggestion = self.suggestions[nextIndex]
            self.selectedSuggestion = nextIndex
            self.displayValue = suggestion
            self.dropEvent = True

    def decSelectedSuggestion(self):
        with self.notify():
            currentIndex = max(-1, self.selectedSuggestion - 1)
            nextIndex = currentIndex % len(self.suggestions)
            nextDisplay = self.suggestions[nextIndex]
            self.displayValue = nextDisplay
            self.selectedSuggestion = nextIndex
            self.dropEvent = True

    def hideSuggestions(self):
        with self.notify():
            self.suggestionsVisible = False

    def showSuggestions(self):
        with self.notify():
            self.generateSuggestions(self.displayValue)
            self.suggestionsVisible = True

    def isShowingSuggestions(self):
        """
        Check if we're currently showing the suggestion dropdown
        by checking if we've made it's height non-zero.
        """
        return self.suggestionsVisible


class ListCtrlComboPopup(wx.ComboPopup):
    """
    This is an empty placeholder to satisfy the interface of
    the ComboCtrl which uses it. All Popup behavior is handled
    inside of `FilterableDropdown`. See its docs for additional
    details.
    """
    def __init__(self):
        wx.ComboPopup.__init__(self)
        self.lc = None

    def Create(self, parent):
        # this ComboCtrl requires a real Wx widget be created
        # thus creating a blank static text object
        self.lc = wx.StaticText(parent)
        return True

    def GetControl(self):
        return self.lc




"""
Demo
"""
import csv
import zipfile
from io import TextIOWrapper

from gooey import Gooey, GooeyParser, PrefixTokenizers, options


print()



with zipfile.ZipFile(r'C:\Users\Chris\Downloads\simplemaps_worldcities_basicv1.71\worldcities.zip') as zf:
    with zf.open('worldcities.csv', 'r') as f:
        reader = csv.reader(TextIOWrapper(f, 'utf-8'))
        data = ['{} - {}'.format(x[4], x[1]) for x in reader][1:][:5000]


@Gooey(program_name='FilterableDropdown Demo', poll_external_updates=True)
def main():
    parser = GooeyParser(description="Example of the Filterable Dropdown")
    parser.add_argument(
        "-a",
        "--myargument",
        metavar='Country',
        help='Search for a country',
        choices=data,
        widget='FilterableDropdown',
        gooey_options=options.FilterableDropdown(**{
            'label_color': (255, 100, 100),
            'placeholder': 'Start typing to view suggestions',
            'search_strategy': options.SearchStrategy(**{
                'type': 'PrefixFilter',
                'choice_tokenizer': PrefixTokenizers.WORDS,
                'input_tokenizer': PrefixTokenizers.REGEX('\s'),
                'ignore_case': True,
                'operator': 'AND',
                'index_suffix': False
            })
        }))
    args = parser.parse_args()
    print(args)


if __name__ == "__main__":
    import sys
    import json
    if 'gooey-seed-ui' in sys.argv:
        print(json.dumps({'-b': data}))
    else:
        main()



import os
import sys
import re

from gooey import Gooey, GooeyParser


@Gooey(advanced=True,  # toggle whether to show advanced config or not
       show_config=True,  # skip config screens all together
       program_name='Вне рамок',  # Defaults to script name
       program_description="Мимо пролетел единорог",
       # Defaults to ArgParse Description
       default_size=(610, 530),  # starting size of the GUI
       required_cols=1,  # number of columns in the "Required" section
       optional_cols=1,  # number of columbs in the "Optional" section
       dump_build_config=False,  # Dump the JSON Gooey uses to configure itself
       load_build_config=None,  # Loads a JSON Gooey-generated configuration
       language = 'russian'
       )
def main():
    parser = GooeyParser(description="""Хотите ли вы купить хотдог?""")
    sg = parser.add_argument_group(
        "", gooey_options={'columns': 1})
    sg.add_argument('--Checkbox',
                    action='store_true',
                    widget="CheckBox",
                    help="Очень важный параметр")

    args = parser.parse_args()

    print("Русский текст, который не кодируется")


if __name__ == "__main__":
    main()


import wx
from gooey import Gooey, GooeyParser

# @Gooey
# def main():
#     parser = GooeyParser(description="Gooey example")
#     parser.add_argument("-a", "--myargument",
#     #                     choices=['one', 'two', 'three'], gooey_options={
#     #                     'label_color': (255, 100, 100)
#     # }
#                         )
#     args = parser.parse_args()
#     print(args)
#     print(args.myargument)
#     print(type(args.myargument))
#
#
# if __name__=="__main__":
#     main()

import wx

class PromptingComboBox(wx.ComboBox) :
    def __init__(self, parent, choices=[], style=0, **par):
        wx.ComboBox.__init__(self, parent, wx.ID_ANY, style=style|wx.CB_DROPDOWN, choices=choices, **par)
        self.choices = choices
        self.Bind(wx.EVT_TEXT, self.OnText)
        self.Bind(wx.EVT_KEY_DOWN, self.OnPress)
        self.ignoreEvtText = False
        self.deleteKey = False

    def OnPress(self, event):
        if event.GetKeyCode() == 8:
            self.deleteKey = True
        event.Skip()

    def OnText(self, event):
        currentText = event.GetString()
        if self.ignoreEvtText:
            self.ignoreEvtText = False
            return
        if self.deleteKey:
            self.deleteKey = False
            if self.preFound:
                currentText =  currentText[:-1]

        self.preFound = False
        for choice in self.choices :
            if choice.startswith(currentText):
                self.ignoreEvtText = True
                self.SetValue(choice)
                self.SetInsertionPoint(len(currentText))
                self.SetTextSelection(len(currentText), len(choice))
                self.preFound = True
                break

class TrialPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, wx.ID_ANY)

        choices = ['grandmother', 'grandfather', 'cousin', 'aunt', 'uncle', 'grandson', 'granddaughter']
        for relative in ['mother', 'father', 'sister', 'brother', 'daughter', 'son']:
            choices.extend(self.derivedRelatives(relative))
        self.choices = choices = sorted(choices)

        mainSizer = wx.FlexGridSizer(2, 2, 5, 10)
        self.SetSizer(mainSizer)

        mainSizer.Add(wx.StaticText(
            self, -1, "Worked in Mac - python 3 - wx phoenix"))
        cb1 = PromptingComboBox(self, choices=choices)
        mainSizer.Add(cb1)

        mainSizer.Add(wx.StaticText(self, -1, "Work arround in Linux-gtk"))
        sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        mainSizer.Add(sizer2)
        filterCtrl = wx.TextCtrl(self, -1, size=(150, -1))
        filterCtrl.Bind(wx.EVT_TEXT, self.OnFilter)
        sizer2.Add(filterCtrl)
        self.cb2 = wx.ComboBox(self, -1, size=(150, -1), choices=choices)
        sizer2.Add(self.cb2)


    def derivedRelatives(self, relative):
        return [relative, 'step' + relative, relative + '-in-law']

    def OnFilter(self, event):
        currentText = event.GetString().upper()
        tmpChoices = [c for c in self.choices if c.startswith(currentText)]
        if tmpChoices != []:
            self.cb2.SetItems(tmpChoices)
            self.cb2.SetValue(tmpChoices[0])
        else:
            self.cb2.SetValue('')
            self.cb2.SetItems([])

if __name__ == '__main__':
    app = wx.App(False)
    frame = wx.Frame (None, -1, 'Demo PromptingComboBox Control and Work around',
                      size=(700, 400))
    TrialPanel(frame)
    frame.Show()
    app.MainLoop()


from gooey import Gooey, GooeyParser

@Gooey
def main():
    parser = GooeyParser(description="Gooey example")
    parser.add_argument("-a", "--myargument", action="store_true")
    args = parser.parse_args()
    print(args)
    print(args.myargument)
    print(type(args.myargument))


if __name__=="__main__":
    main()


import argparse
from gooey import Gooey
from examples.issue458.resource import ConstantArray

def get_resource(num):
    return ConstantArray[num]

@Gooey
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("ResourceNumber")
    args = parser.parse_args()

    print(get_resource(int(args.ResourceNumber)))

if __name__ == "__main__":
    main()


from __future__ import absolute_import

from .textfield import TextField
from .textarea import Textarea
from .password import PasswordField
from .command import CommandField
from .dropdown import Dropdown
from .listbox import Listbox
from .checkbox import CheckBox
from .checkbox import BlockCheckbox
from .counter import Counter
from .radio_group import RadioGroup
from .choosers import *
from .dropdown_filterable import FilterableDropdown
from .numeric_fields import IntegerField, DecimalField
from .slider import Slider



import wx

import wx.lib.inspection
from gooey.gui.components.widgets.textfield import TextField
from gooey.gui.components.widgets.textarea import Textarea
from gooey.gui.components.widgets.password import PasswordField
from gooey.gui.components.widgets.choosers import FileChooser, FileSaver, DirChooser, DateChooser
from gooey.gui.components.widgets.dropdown import Dropdown
from gooey.gui.components.widgets.listbox import Listbox


class CCC(wx.Frame):
    def __init__(self, *args, **kwargs):
        super(CCC, self).__init__(*args, **kwargs)
        x = {'data':{'choices':['one', 'tw'], 'display_name': 'foo', 'help': 'bar', 'commands': ['-t']}, 'id': 1, 'options': {}}

        a = TextField(self, x)
        c = Textarea(self, x)
        b = PasswordField(self, x)
        d = DirChooser(self, x)
        e = FileChooser(self,x)
        f = FileSaver(self, x)
        g = DateChooser(self, x)
        h = Dropdown(self, x)
        i = Listbox(self, x)

        s = wx.BoxSizer(wx.VERTICAL)
        s.Add(a, 0, wx.EXPAND)
        s.Add(b, 0, wx.EXPAND)
        s.Add(c, 0, wx.EXPAND)
        s.Add(d, 0, wx.EXPAND)
        s.Add(e, 0, wx.EXPAND)
        s.Add(f, 0, wx.EXPAND)
        s.Add(g, 0, wx.EXPAND)
        s.Add(h, 0, wx.EXPAND)
        s.Add(i, 0, wx.EXPAND)

        self.SetSizer(s)





app = wx.App()

frame = CCC(None, -1, 'simple.py')
frame.Show()

app.MainLoop()


# import wx
#
# class MainWindow(wx.Frame):
#     def __init__(self, *args, **kwargs):
#         wx.Frame.__init__(self, *args, **kwargs)
#
#         self.panel = wx.Panel(self)
#
#         self.label = wx.StaticText(self.panel, label="Label")
#         self.text = wx.TextCtrl(self.panel)
#         self.button = wx.Button(self.panel, label="Test")
#
#         self.button1 = wx.Button(self.panel, label="ABOVE")
#         self.button2 = wx.Button(self.panel, label="BELLOW")
#
#         self.horizontal = wx.BoxSizer()
#         self.horizontal.Add(self.label, flag=wx.CENTER)
#         self.horizontal.Add(self.text, proportion=1, flag=wx.CENTER)
#         self.horizontal.Add(self.button, flag=wx.CENTER)
#
#         self.vertical = wx.BoxSizer(wx.VERTICAL)
#         self.vertical.Add(self.button1, flag=wx.EXPAND)
#         self.vertical.Add(self.horizontal, proportion=1, flag=wx.EXPAND)
#         self.vertical.Add(self.button2, flag=wx.EXPAND)
#
#         self.panel.SetSizerAndFit(self.vertical)
#         self.Show()
#
#
# app = wx.App(False)
# win = MainWindow(None)
# app.MainLoop()



from gooey.gui.lang.i18n import _

import wx

from gooey.gui.three_to_four import Constants


class BaseDialog(wx.Dialog):
  """
    Common base for CalendarDlg and TimeDlg.
  """
  def __init__(self, parent, pickerClass, pickerGetter, localizedPickerLabel):
    wx.Dialog.__init__(self, parent, title=localizedPickerLabel)

    self.SetBackgroundColour('#ffffff')

    self.ok_button = wx.Button(self, wx.ID_OK, label=_('ok'))
    self.picker = pickerClass(self, style=Constants.WX_DP_DROPDOWN)
    self.pickerGetter = pickerGetter

    vertical_container = wx.BoxSizer(wx.VERTICAL)
    vertical_container.AddSpacer(10)
    vertical_container.Add(self.picker, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 15)

    vertical_container.AddSpacer(10)
    button_sizer = wx.BoxSizer(wx.HORIZONTAL)
    button_sizer.AddStretchSpacer(1)
    button_sizer.Add(self.ok_button, 0)

    vertical_container.Add(button_sizer, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 15)
    vertical_container.AddSpacer(20)
    self.SetSizerAndFit(vertical_container)

    self.Bind(wx.EVT_BUTTON, self.onOkButton, self.ok_button)

  def onOkButton(self, event):
    self.EndModal(wx.ID_OK)
    event.Skip()

  def onCancelButton(self, event):
    try:
      return None
    except:
      self.Close()

  def GetPath(self):
    """
      Return the value chosen in the picker.
      The method is called GetPath() instead of getPath() to emulate the WX Pickers API.
      This allows the Chooser class to work same way with native WX dialogs or childs of BaseDialog.
    """

    return self.pickerGetter(self.picker)






import sys
import unittest
from argparse import ArgumentParser
from collections import namedtuple
from unittest.mock import patch
from unittest.mock import MagicMock

from python_bindings import constants
from tests.harness import instrumentGooey

from gooey.tests import *

class TestGooeyApplication(unittest.TestCase):

    def testFullscreen(self):
        parser = self.basicParser()
        for shouldShow in [True, False]:
            with self.subTest('Should set full screen: {}'.format(shouldShow)):
                with instrumentGooey(parser, fullscreen=shouldShow) as (app, gapp):
                    self.assertEqual(gapp.IsFullScreen(), shouldShow)


    @patch("gui.containers.application.modals.confirmForceStop")
    def testGooeyRequestsConfirmationWhenShowStopWarningModalTrue(self, mockModal):
        """
        When show_stop_warning=False, Gooey should immediately kill the
        running program without additional user confirmation.

        Otherwise, Gooey should show a confirmation modal and, dependending on the
        user's choice, either do nothing or kill the running program.
        """
        Case = namedtuple('Case', ['show_warning', 'shouldSeeConfirm', 'userChooses', 'shouldHaltProgram'])
        testcases = [
            Case(show_warning=True, shouldSeeConfirm=True, userChooses=True, shouldHaltProgram=True),
            Case(show_warning=True, shouldSeeConfirm=True, userChooses=False, shouldHaltProgram=False),
            Case(show_warning=False, shouldSeeConfirm=False, userChooses='N/A', shouldHaltProgram=True),
        ]

        for case in testcases:
            mockModal.reset_mock()
            parser = self.basicParser()
            with instrumentGooey(parser, show_stop_warning=case.show_warning) as (app, gapp):
                mockClientRunner = MagicMock()
                mockModal.return_value = case.userChooses
                gapp.clientRunner = mockClientRunner

                gapp.onStopExecution()

                if case.shouldSeeConfirm:
                    mockModal.assert_called()
                else:
                    mockModal.assert_not_called()

                if case.shouldHaltProgram:
                    mockClientRunner.stop.assert_called()
                else:
                    mockClientRunner.stop.assert_not_called()

    @patch("gui.containers.application.modals.confirmForceStop")
    def testOnCloseShutsDownActiveClients(self, mockModal):
        """
        Issue 592: Closing the UI should clean up any actively running programs
        """
        parser = self.basicParser()
        with instrumentGooey(parser) as (app, gapp):
            gapp.clientRunner = MagicMock()
            gapp.destroyGooey = MagicMock()
            # mocking that the user clicks "yes shut down" in the warning modal
            mockModal.return_value = True
            gapp.onClose()

            mockModal.assert_called()
            gapp.destroyGooey.assert_called()


    def testTerminalColorChanges(self):
        ## Issue #625 terminal panel color wasn't being set due to a typo
        parser = self.basicParser()
        expectedColors = [(255, 0, 0, 255), (255, 255, 255, 255), (100, 100, 100,100)]
        for expectedColor in expectedColors:
            with instrumentGooey(parser, terminal_panel_color=expectedColor) as (app, gapp):
                foundColor = gapp.console.GetBackgroundColour()
                self.assertEqual(tuple(foundColor), expectedColor)


    def testFontWeightsGetSet(self):
        ## Issue #625 font weight wasn't being correctly passed to the terminal
        for weight in [constants.FONTWEIGHT_LIGHT, constants.FONTWEIGHT_BOLD]:
            parser = self.basicParser()
            with instrumentGooey(parser, terminal_font_weight=weight) as (app, gapp):
                terminal = gapp.console.textbox
                self.assertEqual(terminal.GetFont().GetWeight(), weight)


    def basicParser(self):
        parser = ArgumentParser()
        parser.add_argument('--foo')
        return parser




if __name__ == '__main__':
    unittest.main()


"""
Integration tests that exercise Gooey's various run modes

WX Python needs to control the main thread. So, in order to simulate a user
running through the system, we have to execute the actual assertions in a
different thread
"""


import argparse
from gooey import Gooey

class ArgumentHandler:

    def __init__( self, progname ):
        args = self.parser( progname )
        print(args)

    @Gooey
    def parser( self, progname ):
        # Initialize the parser
        parser = argparse.ArgumentParser(prog = progname, description = 'hai')

        # Arguments list
        parser.add_argument( '-x', '--file', help='Some field')

        args = parser.parse_args()

        return args

if __name__ == '__main__':
    ArgumentHandler("foo")


from gooey import Gooey, GooeyParser, gui




email = """
Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
Section 1.10.32 of "de Finibus Bonorum et Malorum", written by Cicero in 45 BC

"Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?"

Section 1.10.33 of "de Finibus Bonorum et Malorum", written by Cicero in 45 BC

"At vero eos et accusamus et iusto odio dignissimos ducimus qui blanditiis praesentium voluptatum deleniti atque corrupti quos dolores et quas molestias excepturi sint occaecati cupiditate non provident, similique sunt in culpa qui officia deserunt mollitia animi, id est laborum et dolorum fuga. Et harum quidem rerum facilis est et expedita distinctio. Nam libero tempore, cum soluta nobis est eligendi optio cumque nihil impedit quo minus id quod maxime placeat facere possimus, omnis voluptas assumenda est, omnis dolor repellendus. Temporibus autem quibusdam et aut officiis debitis aut rerum necessitatibus saepe eveniet ut et voluptates repudiandae sint et molestiae non recusandae. Itaque earum rerum hic tenetur a sapiente delectus, ut aut reiciendis voluptatibus maiores alias consequatur aut perferendis doloribus asperiores repellat."
1914 translation by H. Rackham
Email Number: [{}]
"""


def email_stream(n):
    return (email.format(i) for i in range(n))


@Gooey(buffer_size=10000)
def main():
    parser = GooeyParser()
    parser.add_argument(
        "some_string",
        action="store",
        default="just press Start",
    )

    args = parser.parse_args()
    print(args, flush=True)

    # print too many messages (causes crash for me around 105k).
    for email in email_stream(60000):
        import time
        time.sleep(.01)
        print(email, flush=True)





if __name__ == "__main__":
    main()


import argparse
import unittest

from gooey import GooeyParser
from gooey.tests import *

class TestParentInheritance(unittest.TestCase):

    def test_parent_arguments_exist_in_child(self):
        """
        Verifies that the parents parameter is honoured.
        """
        base_parser = GooeyParser(add_help=False)
        base_parser.add_argument("a_file", widget="FileChooser")

        parser = GooeyParser(parents=[base_parser])
        parser.add_argument("b_file", widget="DirChooser")

        found = 0
        for action in parser._actions:
            if action.dest == "a_file":
                found += 1
            elif action.dest == "b_file":
                found += 1

        self.assertEqual(2, found, "Did not find 2 expected arguments, found " + str(found))
        self.assertEqual(parser.widgets["a_file"], "FileChooser")
        self.assertEqual(parser.widgets["b_file"], "DirChooser")

    def test_parent_arguments_are_not_overridden(self):
        """
        Verifies that the same named argument in a parent and child parser is accepted, and only the child
        parser survives.
        """
        # Verify how vanilla argparse works
        base_parser = argparse.ArgumentParser(add_help=False)
        action1 = base_parser.add_argument("a_file", default="a")

        parser = argparse.ArgumentParser(parents=[base_parser])
        action2 = parser.add_argument("a_file", default="b")

        self._verify_duplicate_parameters(action1, action2, parser)
        # So a child can't override a parent - this isn't textbook inheritance

        # Run the same test on GooeyParser
        base_parser = GooeyParser(add_help=False)
        action1 = base_parser.add_argument("a_file", widget="FileChooser", default="a")

        parser = GooeyParser(parents=[base_parser])
        action2 = parser.add_argument("a_file", widget="DirChooser", default="b")

        self._verify_duplicate_parameters(action1, action2, parser)
        self.assertEqual(parser.widgets["a_file"], "FileChooser")

    def test_duplicates_on_same_parser_are_ignored(self):
        """
        Verify that adding duplicate named arguments works the same in argparse and Gooey.
        Assuming the behaviour of the "default" parameter is a good match for the "widget" parameter.
        """

        # Verify how vanilla argparse works
        parser = argparse.ArgumentParser()
        action1 = parser.add_argument("a_file", default="a")
        action2 = parser.add_argument("a_file", default="b")

        self._verify_duplicate_parameters(action1, action2, parser)

        # Run the same test on GooeyParser
        parser = GooeyParser()
        action1 = parser.add_argument("a_file", default="a", widget="FileChooser")
        action2 = parser.add_argument("a_file", default="b", widget="DirChooser")

        self._verify_duplicate_parameters(action1, action2, parser)
        self.assertEqual(parser.widgets["a_file"], "FileChooser")

    def _verify_duplicate_parameters(self, action1, action2, parser):
        """
        Verify two parameters named a_file exist and the default value is "a".
        """
        found = 0
        for action in parser._actions:
            if action.dest == "a_file":
                found += 1
        self.assertEqual(2, found, "Expected a both actions handling a_file but got " + str(found))
        self.assertEqual(parser.get_default("a_file"), "a")
        self.assertNotEqual(action1, action2)


import time
import unittest

from gooey.gui.lang.i18n import _
from tests.integration.programs import auto_start as auto_start_module


class TestGooeyIntegration(unittest.TestCase):

    def test__gooeyAutoStart(self):
        """Verifies that issue #201 doesn't regress and auto_start skips the config
        screen and hops right into the client's program"""
        from gooey.tests.integration import runner
        runner.run_integration(auto_start_module, self.verifyAutoStart, auto_start=True)

    def verifyAutoStart(self, app, buildSpec):
        """
        When the auto_start flag == True Gooey should skip the
        configuration screen
        """
        time.sleep(1)
        try:
            # Gooey should NOT be showing the name/description headers
            # present on the config page
            title = app.TopWindow.header._header.GetLabel()
            subtitle = app.TopWindow.header._subheader.GetLabel()
            self.assertNotEqual(title, buildSpec['program_name'])
            self.assertNotEqual(subtitle, buildSpec['program_description'])

            # Gooey should be showing the console messages straight away
            # without manually starting the program
            title = app.TopWindow.header._header.GetLabel()
            subtitle = app.TopWindow.header._subheader.GetLabel()
            self.assertEqual(title,_("running_title"))
            self.assertEqual(subtitle, _('running_msg'))

            # Wait for Gooey to swap the header to the final screen
            while app.TopWindow.header._header.GetLabel() == _("running_title"):
                time.sleep(.1)

            # verify that we've landed on the success screen
            title = app.TopWindow.header._header.GetLabel()
            subtitle = app.TopWindow.header._subheader.GetLabel()
            self.assertEqual(title, _("finished_title"))
            self.assertEqual(subtitle, _('finished_msg'))


            # and that output was actually written to the console
            self.assertIn("Success", app.TopWindow.console.textbox.GetValue())
        except:
            app.TopWindow.Destroy()
            raise
        else:
            import wx
            wx.CallAfter(app.TopWindow.Destroy)
            return None




if __name__ == '__main__':
    unittest.main()









import wx


class PromptingComboBox(wx.ComboBox):
    def __init__(self, parent, value, choices=[], style=0, **par):
        wx.ComboBox.__init__(self, parent, wx.ID_ANY, value, style=style | wx.CB_DROPDOWN,
                             choices=choices, **par)
        self.choices = choices
        self.Bind(wx.EVT_TEXT, self.EvtText)
        self.Bind(wx.EVT_CHAR, self.EvtChar)
        self.Bind(wx.EVT_COMBOBOX, self.EvtCombobox)
        self.ignoreEvtText = False

    def EvtCombobox(self, event):
        self.ignoreEvtText = True
        event.Skip()

    def EvtChar(self, event):
        if event.GetKeyCode() == 8:
            self.ignoreEvtText = True
        event.Skip()

    def EvtText(self, event):
        if self.ignoreEvtText:
            self.ignoreEvtText = False
            return
        currentText = event.GetString()
        found = False
        for choice in self.choices:
            if choice.startswith(currentText):
                self.ignoreEvtText = True
                self.SetValue(choice)
                self.SetInsertionPoint(len(currentText))
                self.SetMark(len(currentText), len(choice))
                found = True
                break
        if not found:
            event.Skip()


class TrialPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, wx.ID_ANY)

        choices = ['grandmother', 'grandfather', 'cousin', 'aunt', 'uncle', 'grandson',
                   'granddaughter']
        for relative in ['mother', 'father', 'sister', 'brother', 'daughter', 'son']:
            choices.extend(self.derivedRelatives(relative))

        cb = PromptingComboBox(self, "default value", choices, style=wx.CB_SORT)

    def derivedRelatives(self, relative):
        return [relative, 'step' + relative, relative + '-in-law']


if __name__ == '__main__':
    app = wx.App()
    frame = wx.Frame(None, -1, 'Demo PromptingComboBox Control', size=(400, 50))
    TrialPanel(frame)
    frame.Show()
    app.MainLoop()


"""
Converts argparse parser actions into json "Build Specs"
"""
import argparse
import json
import os
import sys
from argparse import (
    _CountAction,
    _HelpAction,
    _StoreConstAction,
    _StoreFalseAction,
    _StoreTrueAction,
    _StoreAction,
    _SubParsersAction)
from collections import OrderedDict
from functools import partial
from uuid import uuid4

from gooey.python_bindings.gooey_parser import GooeyParser
from gooey.util.functional import merge, getin, identity, assoc
from gooey.gui.components.options.validators import validators
from gooey.gui.components.options.validators import collect_errors

VALID_WIDGETS = (
    'FileChooser',
    'MultiFileChooser',
    'FileSaver',
    'DirChooser',
    'DateChooser',
    'TimeChooser',
    'TextField',
    'Dropdown',
    'Counter',
    'RadioGroup',
    'CheckBox',
    'BlockCheckbox',
    'MultiDirChooser',
    'Textarea',
    'PasswordField',
    'Listbox',
    'FilterableDropdown',
    'IntegerField',
    'DecimalField',
    'Slider'
)


# TODO: validate Listbox. When required, nargs must be +

class UnknownWidgetType(Exception):
    pass


class UnsupportedConfiguration(Exception):
    pass



# TODO: merge the default foreground and bg colors from the
# baseline build_spec
item_default = {
    'error_color': '#ea7878',
    'label_color': '#000000',
    'help_color': '#363636',
    'full_width': False,
    'validator': {
        'type': 'local',
        'test': 'lambda x: True',
        'message': ''
    },
    'external_validator': {
        'cmd': '',
    }
}


def convert(parser, **kwargs):
    """
    Converts a parser into a JSON representation

    TODO:
    This is in desperate need of refactor. It wasn't build with supporting
    all (or any) of this configuration in mind. The use of global defaults
    are actively getting in the way of easily adding more configuration options.

    Pain points:
        - global data sprinkled throughout the calls
        - local data threaded through calls
        - totally unclear what the data structures even hold
        - everything is just mushed together and gross. unwinding argparse also
          builds validators, handles coercion, and so on...
        - converts to an entirely bespoke json mini-language that mirrors
          the internal structure of argparse.
    Refactor plan:
        - Investigate restructuring the core data representation. As is, it is ad-hoc
          and largely tied to argparse's goofy internal structure. May be worth moving to
          something "standard." Though, not sure what the options are.
        - standardize how these things read from the environment. No global in some local in others.
        - Investigate splitting the whole thing into phases (ala Ring). Current thinking is that
          a lot of this stuff could be modelled more like pluggable upgrades to the base structure.
        - I want to add a helpful validation stage to catch user errors like invalid gooey_options
    """

    group_defaults = {
        'legacy': {
            'required_cols': kwargs['num_required_cols'],
            'optional_cols': kwargs['num_optional_cols']
        },
        'columns': 2,
        'padding': 10,
        'show_border': False
    }

    assert_subparser_constraints(parser)
    x = {
        'layout': 'standard',
        'widgets': OrderedDict(
            (choose_name(name, sub_parser), {
                'command': name,
                'name': choose_name(name, sub_parser),
                'help': get_subparser_help(sub_parser),
                'description': '',
                'contents': process(sub_parser,
                                    getattr(sub_parser, 'widgets', {}),
                                    getattr(sub_parser, 'options', {}),
                                    group_defaults)
            }) for name, sub_parser in iter_parsers(parser))
    }

    if kwargs.get('use_legacy_titles'):
        return apply_default_rewrites(x)
    return x


def process(parser, widget_dict, options, group_defaults):
    mutex_groups = parser._mutually_exclusive_groups
    raw_action_groups = [extract_groups(group, group_defaults) for group in parser._action_groups
                         if group._group_actions]
    corrected_action_groups = reapply_mutex_groups(mutex_groups, raw_action_groups)

    return categorize2(strip_empty(corrected_action_groups), widget_dict, options)


def strip_empty(groups):
    return [group for group in groups if group['items']]


def assert_subparser_constraints(parser):
    if has_subparsers(parser._actions):
        if has_required(parser._actions):
            raise UnsupportedConfiguration(
                "Gooey doesn't currently support top level required arguments "
                "when subparsers are present.")


def iter_parsers(parser):
    ''' Iterate over name, parser pairs '''
    try:
        return get_subparser(parser._actions).choices.items()
    except:
        return iter([('::gooey/default', parser)])


def get_subparser_help(parser):
    if isinstance(parser, GooeyParser):
        return getattr(parser.parser, 'usage', '')
    else:
        return getattr(parser, 'usage', '')


def extract_groups(action_group, group_defaults):
    '''
    Recursively extract argument groups and associated actions
    from ParserGroup objects
    '''
    return {
        'name': action_group.title,
        'description': action_group.description,
        'items': [action for action in action_group._group_actions
                  if not is_help_message(action)],
        'groups': [extract_groups(group, group_defaults)
                   for group in action_group._action_groups],
        'options': handle_option_merge(
            group_defaults,
            getattr(action_group, 'gooey_options', {}),
            action_group.title)
    }


def handle_option_merge(group_defaults, incoming_options, title):
    """
    Merges a set of group defaults with incoming options.

    A bunch of ceremony here is to ensure backwards compatibility with the old
    num_required_cols and num_optional_cols decorator args. They are used as
    the seed values for the new group defaults which keeps the old behavior
    _mostly_ in tact.

    Known failure points:
        * Using custom groups / names. No 'positional arguments' group
          means no required_cols arg being honored
        * Non-positional args marked as required. It would take group
          shuffling along the lines of that required to make
          mutually exclusive groups show in the correct place. In short, not
          worth the complexity for a legacy feature that's been succeeded by
          a much more powerful alternative.
    """
    if title == 'positional arguments':
        # the argparse default 'required' bucket
        req_cols = getin(group_defaults, ['legacy', 'required_cols'], 2)
        new_defaults = assoc(group_defaults, 'columns', req_cols)
        return merge(new_defaults, incoming_options)
    else:
        opt_cols = getin(group_defaults, ['legacy', 'optional_cols'], 2)
        new_defaults = assoc(group_defaults, 'columns', opt_cols)
        return merge(new_defaults, incoming_options)



def apply_default_rewrites(spec):
    top_level_subgroups = list(spec['widgets'].keys())

    for subgroup in top_level_subgroups:
        path = ['widgets', subgroup, 'contents']
        contents = getin(spec, path)
        for group in contents:
            if group['name'] == 'positional arguments':
                group['name'] = 'required_args_msg'
            if group['name'] == 'optional arguments':
                group['name'] = 'optional_args_msg'
    return spec


def contains_actions(a, b):
    ''' check if any actions(a) are present in actions(b) '''
    return set(a).intersection(set(b))


def reapply_mutex_groups(mutex_groups, action_groups):
    # argparse stores mutually exclusive groups independently
    # of all other groups. So, they must be manually re-combined
    # with the groups/subgroups to which they were originally declared
    # in order to have them appear in the correct location in the UI.
    #
    # Order is attempted to be preserved by inserting the MutexGroup
    # into the _actions list at the first occurrence of any item
    # where the two groups intersect
    def swap_actions(actions):
        for mutexgroup in mutex_groups:
            mutex_actions = mutexgroup._group_actions
            if contains_actions(mutex_actions, actions):
                # make a best guess as to where we should store the group
                targetindex = actions.index(mutexgroup._group_actions[0])
                # insert the _ArgumentGroup container
                actions[targetindex] = mutexgroup
                # remove the duplicated individual actions
                actions = [action for action in actions
                        if action not in mutex_actions]
        return actions

    return [group.update({'items': swap_actions(group['items'])}) or group
            for group in action_groups]


def categorize2(groups, widget_dict, options):
    defaults = {'label_color': '#000000', 'description_color': '#363636'}
    return [{
        'name': group['name'],
        'items': list(categorize(group['items'], widget_dict, options)),
        'groups': categorize2(group['groups'], widget_dict, options),
        'description': group['description'],
        'options': merge(defaults ,group['options'])
    } for group in groups]


def categorize(actions, widget_dict, options):
    _get_widget = partial(get_widget, widget_dict)
    for action in actions:

        if is_mutex(action):
            yield build_radio_group(action, widget_dict, options)

        elif is_standard(action):
            yield action_to_json(action, _get_widget(action, 'TextField'), options)
        
        elif is_file(action):
            yield action_to_json(action, _get_widget(action, 'FileSaver'), options)

        elif is_choice(action):
            yield action_to_json(action, _get_widget(action, 'Dropdown'), options)

        elif is_flag(action):
            yield action_to_json(action, _get_widget(action, 'CheckBox'), options)

        elif is_counter(action):
            _json = action_to_json(action, _get_widget(action, 'Counter'), options)
            # pre-fill the 'counter' dropdown
            _json['data']['choices'] = list(map(str, range(0, 11)))
            yield _json
        else:
            raise UnknownWidgetType(action)


def get_widget(widgets, action, default):
    supplied_widget = widgets.get(action.dest, None)
    return supplied_widget or default


def is_required(action):
    '''
    _actions possessing the `required` flag and not implicitly optional
    through `nargs` being '*' or '?'
    '''
    return not isinstance(action, _SubParsersAction) and (
    action.required == True and action.nargs not in ['*', '?'])


def is_mutex(action):
    return isinstance(action, argparse._MutuallyExclusiveGroup)


def has_required(actions):
    return list(filter(None, list(filter(is_required, actions))))


def is_subparser(action):
    return isinstance(action, _SubParsersAction)


def has_subparsers(actions):
    return list(filter(is_subparser, actions))


def get_subparser(actions):
    return list(filter(is_subparser, actions))[0]


def is_optional(action):
    '''
    _actions either not possessing the `required` flag or implicitly optional
    through `nargs` being '*' or '?'
    '''
    return (not action.required) or action.nargs in ['*', '?']


def is_choice(action):
    ''' action with choices supplied '''
    return action.choices

def is_file(action):
    ''' action with FileType '''
    return isinstance(action.type, argparse.FileType)

def is_standard(action):
    """ actions which are general "store" instructions.
    e.g. anything which has an argument style like:
       $ script.py -f myfilename.txt
    """
    boolean_actions = (
        _StoreConstAction, _StoreFalseAction,
        _StoreTrueAction
    )
    return (not action.choices
            and not isinstance(action.type, argparse.FileType)
            and not isinstance(action, _CountAction)
            and not isinstance(action, _HelpAction)
            and type(action) not in boolean_actions)


def is_flag(action):
    """ _actions which are either storeconst, store_bool, etc.. """
    action_types = [_StoreTrueAction, _StoreFalseAction, _StoreConstAction]
    return any(list(map(lambda Action: isinstance(action, Action), action_types)))


def is_counter(action):
    """ _actions which are of type _CountAction """
    return isinstance(action, _CountAction)


def is_default_progname(name, subparser):
    return subparser.prog == '{} {}'.format(os.path.split(sys.argv[0])[-1], name)


def is_help_message(action):
    return isinstance(action, _HelpAction)


def choose_name(name, subparser):
    return name if is_default_progname(name, subparser) else subparser.prog


def build_radio_group(mutex_group, widget_group, options):
  return {
    'id': str(uuid4()),
    'type': 'RadioGroup',
    'cli_type': 'optional',
    'group_name': 'Choose Option',
    'required': mutex_group.required,
    'options': merge(item_default, getattr(mutex_group, 'gooey_options', {})),
    'data': {
      'commands': [action.option_strings for action in mutex_group._group_actions],
      'widgets': list(categorize(mutex_group._group_actions, widget_group, options))
    }
  }


def action_to_json(action, widget, options):
    dropdown_types = {'Listbox', 'Dropdown', 'Counter'}
    if action.required:
        # Text fields get a default check that user input is present
        # and not just spaces, dropdown types get a simplified
        # is-it-present style check
        validator = ('user_input and not user_input.isspace()'
                     if widget not in dropdown_types
                     else 'user_input')
        error_msg = 'This field is required'
    else:
        # not required; do nothing;
        validator = 'True'
        error_msg = ''

    base = merge(item_default, {
        'validator': {
            'type': 'ExpressionValidator',
            'test': validator,
            'message': error_msg
        },
    })

    default = handle_default(action, widget)
    if default == argparse.SUPPRESS:
        default = None

    final_options = merge(base, options.get(action.dest) or {})
    validate_gooey_options(action, widget, final_options)

    return {
        'id': action.option_strings[0] if action.option_strings else action.dest,
        'type': widget,
        'cli_type': choose_cli_type(action),
        'required': action.required,
        'data': {
            'display_name': action.metavar or action.dest,
            'help': action.help,
            'required': action.required,
            'nargs': action.nargs or '',
            'commands': action.option_strings,
            'choices': list(map(str, action.choices)) if action.choices else [],
            'default': default,
            'dest': action.dest,
        },
        'options': final_options
    }


def validate_gooey_options(action, widget, options):
    """Very basic field validation / sanity checking for
    the time being.

    Future plans are to assert against the options and actions together
    to facilitate checking that certain options like `initial_selection` in
    RadioGroups map to a value which actually exists (rather than exploding
    at runtime with an unhelpful error)

    Additional problems with the current approach is that no feedback is given
    as to WHERE the issue took place (in terms of stacktrace). Which means we should
    probably explode in GooeyParser proper rather than trying to collect all the errors here.
    It's not super ideal in that the user will need to run multiple times to
    see all the issues, but, ultimately probably less annoying that trying to
    debug which `gooey_option` key had an issue in a large program.

    That said "better is the enemy of done." This is good enough for now. It'll be
    a TODO: better validation 
    """
    errors = collect_errors(validators, options)
    if errors:
        from pprint import pformat
        raise ValueError(str(action.dest) + str(pformat(errors)))


def choose_cli_type(action):
    return 'positional' \
            if action.required and not action.option_strings \
            else 'optional'


def coerce_default(default, widget):
    """coerce a default value to the best appropriate type for
    ingestion into wx"""
    dispatcher = {
        'Listbox': clean_list_defaults,
        'Dropdown': safe_string,
        'Counter': safe_string
    }

    # Issue #321:
    # Defaults for choice types must be coerced to strings
    # to be able to match the stringified `choices` used by `wx.ComboBox`
    cleaned = clean_default(default)

    # dispatch to the appropriate cleaning function, or return the value
    # as is if no special handler is present
    return dispatcher.get(widget, identity)(cleaned)


def handle_default(action, widget):
    handlers = [
        [textinput_with_nargs_and_list_default, coerse_nargs_list],
        [is_widget('Listbox'), clean_list_defaults],
        [is_widget('Dropdown'), coerce_str],
        [is_widget('Counter'), safe_string]
    ]
    for matches, apply_coercion in handlers:
        if matches(action, widget):
            return apply_coercion(action.default)
    return clean_default(action.default)


def coerse_nargs_list(default):
    """
    nargs=* and defaults which are collection types
    must be transformed into a CLI equivalent form. So, for
    instance, ['one two', 'three'] => "one two" "three"

    This only applies when the target widget is a text input. List
    based widgets such as ListBox should keep their defaults in list form

    Without this transformation, `add_arg('--foo', default=['a b'], nargs='*')` would show up in
    the UI as the literal string `['a b']` brackets and all.
    """
    return ' '.join('"{}"'.format(x) for x in default)

def is_widget(name):
    def equals(action, widget):
        return widget == name
    return equals


def textinput_with_nargs_and_list_default(action, widget):
    """
    Vanilla TextInputs which have nargs options which produce lists (i.e.
    nargs +, *, N, or REMAINDER) need to have their default values transformed
    into CLI style space-separated entries when they're supplied as a list of values
    on the Python side.
    """
    return (
        widget in {'TextField', 'Textarea', 'PasswordField'}
        and (isinstance(action.default, list) or isinstance(action.default, tuple))
        and is_list_based_nargs(action))


def is_list_based_nargs(action):
    """ """
    return isinstance(action.nargs, int) or action.nargs in {'*', '+', '...'}



def clean_list_defaults(default_values):
    """
    Listbox's default's can be passed as a single value
    or a collection of values (due to multiple selections). The list interface
    is standardized on for ease.
    """
    wrapped_values = ([default_values]
                      if isinstance(default_values, str)
                      else default_values)
    return [safe_string(value) for value in wrapped_values or []]


def clean_default(default):
    """
    Attempts to safely coalesce the default value down to
    a valid JSON type.
    """
    try:
        json.dumps(default)
        return default
    except TypeError as e:
        # see: Issue #377
        # if is ins't json serializable (i.e. primitive data) there's nothing
        # useful for Gooey to do with it (since Gooey deals in primitive data
        # types). So the correct behavior is dropping them. This affects ONLY
        # gooey, not the client code.
        return None


def safe_string(value):
    """
    Coerce a type to string as long as it isn't None or Boolean
    TODO: why do I have this special boolean case..?
    """
    if value is None or isinstance(value, bool):
        return value
    else:
        return str(value)


def coerce_str(value):
    """
    Coerce the incoming type to string as long as it isn't None
    """
    return str(value) if value is not None else value



def this_is_a_comment(action, widget):
    """
    TODO:
     - better handling of nargs.
     - allowing a class of "Narg'able" widget variants that allow dynamically adding options.
    Below are some rough notes on the current widgets and their nargs behavior (or lack of)
    """

    asdf = [
        # choosers are all currently treated as
        # singular inputs regardless of nargs status.
        'FileChooser',
        'MultiFileChooser',
        'FileSaver',
        'DirChooser',
        'DateChooser',
        'TimeChooser',

        # radiogroup requires no special logic. Delegates to internal widgets
        'RadioGroup',
        # nargs invalid
        'CheckBox',
        # nargs invalid
        'BlockCheckbox',

        # currently combines everything into a single, system _sep separated string
        # potential nargs behavior
        #   input: - each item gets a readonly textbox?
        #          - each item is its own editable widget?
        #          - problem with this idea is selected a ton of files would break the UI.
        #            maybe a better option would be to expose what's been added as a small
        #            list view? That way its a fixed size even if they choose 100s of files.
        #
        'MultiDirChooser',
        # special case. convert default to list of strings
        'Listbox',

        # special cases. coerce default to string
        'Dropdown',
        'Counter',

        # nargs behavior:
        #   - convert to space separated list of strings
        'TextField',
        'Textarea',
        'PasswordField',
    ]


"""
Dynamic updates in Gooey

Simple example demonstrating feeding Gooey a dynamic list of file names at runtime
"""
import datetime
import json
import os
import sys

from gooey import Gooey, GooeyParser


directory_error = (
    'Unable to create a folder for your save files!\n'
    'Make sure you\re running in a directory where you have write permissions!'
)

@Gooey(
    program_name="Using Dynamic Values",
    poll_external_updates=True
)
def main():
    parser = GooeyParser(description='An example of polling for updates at runtime')
    g = parser.add_argument_group()
    stuff = g.add_mutually_exclusive_group(
        required=True,
        gooey_options={
            'initial_selection': 0
        }
    )
    stuff.add_argument(
        '--save',
        metavar='Save Progress',
        action='store_true',
        help='Take a snap shot of your current progress!'
    )
    stuff.add_argument(
        '--load',
        metavar='Load Previous Save',
        help='Load a Previous save file',
        dest='filename',
        widget='Listbox',
        nargs='+',
        choices=list_savefiles(),
        gooey_options={
            'validator': {
                'test': 'user_input != "Select Option"',
                'message': 'Choose a save file from the list'
            }
        }
    )

    args = parser.parse_args()

    if args.save:
        save_file()
    else:
        for f in args.filename:
            read_file(os.path.join('saves', f))
            print('Finished reading file %s!' % f)


def read_file(path):
    with open(path, 'r') as f:
        print(f.read())


def list_savefiles():
    """ List all available files in the save directory """
    return list(sorted(os.listdir('saves'), reverse=True))


def show_error_modal(error_msg):
    """ Spawns a modal with error_msg"""
    # wx imported locally so as not to interfere with Gooey
    import wx
    app = wx.App()
    dlg = wx.MessageDialog(None, error_msg, 'Error', wx.ICON_ERROR)
    dlg.ShowModal()
    dlg.Destroy()


def save_file():
    """Save a plain ol' text file with some metadata in the title"""
    next_number = '{:04d}'.format(len(list_savefiles()) + 1)
    now = datetime.datetime.now().strftime('%b %d, %Y - %H.%M%p')
    filename = 'Save{}{}.save'.format(next_number, now)
    with open(os.path.join('saves', filename), 'w') as f:
        f.write("Hello World!")
    print('Saved {}!'.format(filename))


def mk_savedir():
    """
    Attempt to create a directory where we can store the user's save files
    """
    try:
        os.mkdir('saves')
    except IOError as e:
        if not e.winerror == 183:  # already exists
            show_error_modal(directory_error)
            sys.exit(1)


if __name__ == '__main__':
    if 'gooey-seed-ui' in sys.argv:
        print(json.dumps({'--load': list_savefiles()}))
    else:
        mk_savedir()
        main()



from gooey import Gooey, GooeyParser
import os

@Gooey(optional_cols=2,
        program_name="Elapsed / Remaining Timer on Progress in Gooey",
        progress_regex=r"^progress: (?P<current>\d+)/(?P<total>\d+)$",
        progress_expr="current / total * 100",
        timing_options = {
                'show_time_remaining':True,
                'hide_time_remaining_on_complete':False
        }
        )
def parse_args():
    prog_descrip = 'Elapsed / Remaining Timer on Progress in Gooey'
    parser = GooeyParser(description=prog_descrip)

    sub_parsers = parser.add_subparsers(help='commands', dest='command')

    range_parser = sub_parsers.add_parser('range')

    range_parser.add_argument('--length',default=10000)

    return parser.parse_args()

def compute_range(length):
    for i in range(10):
        import time
        from random import randint
        time.sleep(.4)
        print(f"progress: {i}/{10}")

if __name__ == '__main__':
    conf = parse_args()
    if conf.command == 'range':
        compute_range(int(conf.length))




# from gooey import Gooey, GooeyParser
# import os
#
#
# @Gooey(progress={
#     'progress_bar': {
#         'regex': r"^progress: (?P<current>\d+)/(?P<total>\d+)$",
#         # could pass the full args namespace here...?
#         # Hmm...
#         'expr': "current / total * 100"
#     },
#     'elapsed': {
#         'show': True,
#         'format': '00:00',
#         'hide_on_completion': False
#     },
#     'estimated_remaining': {
#         'show': True,
#         'format': '00:00',
#         'hide_on_completion': False
#     }
# })
#
#
# @Gooey(optional_cols=2,
#        program_name="Elapsed / Remaining Timer on Progress in Gooey",
#        progress_regex=r"^progress: (?P<current>\d+)/(?P<total>\d+)$",
#        progress_expr="current / total * 100",
#        hide_progress_msg=True,
#        timing={
#            'elapsed': {
#                'show': True,
#                'format': '00:00',
#                'hide_on_completion': False
#            },
#            'estimated_remaining': {
#                'show': True,
#                'format': '00:00',
#                'hide_on_completion': False
#            }
#        }
#        )
# def parse_args():
#     prog_descrip = 'Elapsed / Remaining Timer on Progress in Gooey'
#     parser = GooeyParser(description=prog_descrip)
#
#     sub_parsers = parser.add_subparsers(help='commands', dest='command')
#
#     range_parser = sub_parsers.add_parser('range')
#
#     range_parser.add_argument('--length',default=50)
#
#     return parser.parse_args()
#
# def compute_range(length):
#
#     for i in range(length):
#         import time
#         print('Action 1: expensive computation', str(time.time()))
#         time.sleep(.2)
#         print(f"progress: {i}/{length}")
#
#     for i in range(length):
#         import time
#         print('Action 2: some log about an expensive computation', str(time.time()))
#         time.sleep(.2)
#         print(f"progress: {i}/{length}")
#
# if __name__ == '__main__':
#     conf = parse_args()
#     if conf.command == 'range':
#         compute_range(int(conf.length))


from contextlib import contextmanager

import time
from threading import Thread

import wx

from gooey.gui import application
from python_bindings.config_generator import create_from_parser
from python_bindings.gooey_decorator import defaults
from util.functional import merge




@contextmanager
def instrumentGooey(parser, **kwargs):
    """
    Context manager used during testing for setup/tear down of the
    WX infrastructure during subTests.

    Weirdness warning: this uses a globally reused wx.App instance.
    """
    from gooey.tests import app
    if app == None:
        raise Exception("App instance has not been created! This is likely due to "
                        "you forgetting to add the magical import which makes all these "
                        "tests work. See the module doc in gooey.tests.__init__ for guidance")
    buildspec = create_from_parser(parser, "", **merge(defaults, kwargs))
    app, gooey = application._build_app(buildspec, app)
    app.SetTopWindow(gooey)
    try:
        yield (app, gooey)
    finally:
        wx.CallAfter(app.ExitMainLoop)
        gooey.Destroy()
        app.SetTopWindow(None)
        del gooey



import argparse

from gooey import Gooey, GooeyParser


@Gooey(program_name='Argument Groups Demo',
       tabbed_groups=True,
       navigation='Tabbed')
def main():
    settings_msg = 'Example program to show how to place multiple argument groups as tabs'
    parser = GooeyParser(description=settings_msg)

    group1 = parser.add_argument_group('General')
    group1.add_argument('--opt1', action='store_true',
                        help='Option one')

    group2 = parser.add_argument_group('Options')
    group2.add_argument('--opt2', action='store_true',
                        help='Option two')

    args=parser.parse_args()

    print("hey")

if __name__ == '__main__':
    main()


import argparse
from gooey import Gooey

@Gooey(dump_build_config=True, advanced=False)
def main():
  parser = argparse.ArgumentParser(description='Process some integers.')
  parser.add_argument('integers', metavar='N', type=int, nargs='+',
                     help='an integer for the accumulator')

  args = parser.parse_args()
  print 'Good job!'

if __name__ == '__main__':
  main()



from gooey import Gooey, GooeyParser
from argparse import ArgumentParser

def some_function(id_, len_):
    print(id_)
    print(len_)
    return

@Gooey(program_name = "Machine Learning Pipeline - Spatial Proximity Check", required_cols = 2)
# define main function to execute user based query
def subprocess(args):
    # pop the filename off the sys.argv list
    pipeline_arg = sys.argv.index('--pipeline_dataset')
    pipeline_val = sys.argv[pipeline_arg + 1]
    # feed it to the rest of your program as usual
    # df = pd.read_csv(dataset)
    # cols = tuple(df.columns)

    parser2 = GooeyParser(description = "Machine Learning Pipeline - Base Data Input")
    parser2.add_argument('--pipeline_dataset', widget='Textarea', default=pipeline_val, required=True, gooey_options={'visible': False})
    parser2.add_argument('id', metavar = 'ID Column Name', widget = 'Dropdown', choices = [pipeline_val])
    parser2.add_argument('length', metavar = 'Length (ft) Column Name', widget = 'Dropdown', choices = [pipeline_val])
    args = parser2.parse_args()
    print(args)

import sys
subprocess(sys.argv)


"""
Module for evaluating time elapsed & time remaining from progress
"""
import wx
from gooey.gui.pubsub import pub
from gooey.gui import events

class Timing(object):

    def __init__(self, parent):
        self.startTime = 0
        self.estimatedRemaining = None
        self.wxTimer = wx.Timer(parent)
        self.parent = parent
        parent.Bind(wx.EVT_TIMER, self.publishTime, self.wxTimer)

        pub.subscribe(events.PROGRESS_UPDATE, self._updateEstimate)

    def _updateEstimate(self, *args, **kwargs):
        prog = kwargs.get('progress')
        if(not prog): 
            self.estimatedRemaining = None
            return
        if(prog > 0):
            self.estimatedRemaining = estimate_time_remaining(prog,self.startTime)

    def publishTime(self, *args, **kwargs):
        pub.send_message(
            events.TIME_UPDATE,
            start=self.startTime,
            current=get_current_time(),
            elapsed_time=format_interval(get_elapsed_time(self.startTime)),
            estimatedRemaining=format_interval(self.estimatedRemaining))

    def start(self):
        self.startTime = get_current_time()
        self.estimatedRemaining = None
        self.wxTimer.Start()

    def stop(self):
        self.wxTimer.Stop()

def format_interval(timeValue):
    """
    Formats a number of seconds as a clock time, [H:]MM:SS
    Parameters
    ----------
    t  : int
        Number of seconds.
    Returns
    -------
    out  : str
        [H:]MM:SS
    """
    # https://github.com/tqdm/tqdm/blob/0cd9448b2bc08125e74538a2aea6af42ee1a7b6f/tqdm/std.py#L228
    try:
        mins, s = divmod(int(timeValue), 60)
        h, m = divmod(mins, 60)
        if h:
            return '{0:d}:{1:02d}:{2:02d}'.format(h, m, s)
        else:
            return '{0:02d}:{1:02d}'.format(m, s)
    except:
        return None

def get_elapsed_time(startTime):
    """
    Get elapsed time in form of seconds. Provide a start time in seconds as float.

    Args:
        startTime (float): Start time to compare against in seconds.

    Returns:
        float: Time between start time and now
    """
    return get_current_time() - startTime

def estimate_time_remaining(progress,startTime):
    # https://github.com/tqdm/tqdm/blob/0cd9448b2bc08125e74538a2aea6af42ee1a7b6f/tqdm/std.py#L392
    # https://github.com/tqdm/tqdm/blob/0cd9448b2bc08125e74538a2aea6af42ee1a7b6f/tqdm/std.py#L417
    _rate = progress / get_elapsed_time(startTime)
    return ((100 - progress) / _rate)

def get_current_time():
    """
    Returns a float of the current time in seconds. Attempt to import perf_counter (more accurate in 3.4+), otherwise utilise timeit.

    Returns:
        float: Current time in seconds from performance counter.
    """
    try:
        from time import perf_counter
        return perf_counter()
    except:
        import timeit
        return timeit.default_timer()



from gooey import Gooey, GooeyParser
from argparse import ArgumentParser

@Gooey()
def main():
    base_parser = GooeyParser(add_help=False)
    base_parser.add_argument("a_file", widget="FileChooser")

    parser = GooeyParser(parents=[base_parser])
    parser.add_argument("a_file", widget="FileChooser")

    print(parser.parse_args())


if __name__ == "__main__":
    main()